# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: protos/annotations/validation.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'protos/annotations/validation.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#protos/annotations/validation.proto\x12\x03Ydb\x1a google/protobuf/descriptor.proto\"\x9b\x01\n\x05Limit\x12!\n\x05range\x18\x01 \x01(\x0b\x32\x10.Ydb.Limit.RangeH\x00\x12\x0c\n\x02lt\x18\x02 \x01(\rH\x00\x12\x0c\n\x02le\x18\x03 \x01(\rH\x00\x12\x0c\n\x02\x65q\x18\x04 \x01(\rH\x00\x12\x0c\n\x02ge\x18\x05 \x01(\rH\x00\x12\x0c\n\x02gt\x18\x06 \x01(\rH\x00\x1a!\n\x05Range\x12\x0b\n\x03min\x18\x01 \x01(\r\x12\x0b\n\x03max\x18\x02 \x01(\rB\x06\n\x04kind\"3\n\x06MapKey\x12\x1a\n\x06length\x18\x01 \x01(\x0b\x32\n.Ydb.Limit\x12\r\n\x05value\x18\x02 \x01(\t:1\n\x08required\x12\x1d.google.protobuf.FieldOptions\x18\xe2\xac\x05 \x01(\x08:9\n\x04size\x12\x1d.google.protobuf.FieldOptions\x18\xe3\xac\x05 \x01(\x0b\x32\n.Ydb.Limit:;\n\x06length\x12\x1d.google.protobuf.FieldOptions\x18\xe4\xac\x05 \x01(\x0b\x32\n.Ydb.Limit:=\n\x07map_key\x12\x1d.google.protobuf.FieldOptions\x18\xe5\xac\x05 \x01(\x0b\x32\x0b.Ydb.MapKey:.\n\x05value\x12\x1d.google.protobuf.FieldOptions\x18\xe6\xac\x05 \x01(\tBG\n\x0etech.ydb.protoZ2github.com/ydb-platform/ydb-go-genproto/protos/Ydb\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.annotations.validation_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016tech.ydb.protoZ2github.com/ydb-platform/ydb-go-genproto/protos/Ydb\370\001\001'
  _globals['_LIMIT']._serialized_start=79
  _globals['_LIMIT']._serialized_end=234
  _globals['_LIMIT_RANGE']._serialized_start=193
  _globals['_LIMIT_RANGE']._serialized_end=226
  _globals['_MAPKEY']._serialized_start=236
  _globals['_MAPKEY']._serialized_end=287
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_auth.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v5.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15protos/ydb_auth.proto\x12\x08Ydb.Auth\x1a\x1aprotos/ydb_operation.proto\"i\n\x0cLoginRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04user\x18\x02 \x01(\t\x12\x10\n\x08password\x18\x03 \x01(\t\"=\n\rLoginResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x1c\n\x0bLoginResult\x12\r\n\x05token\x18\x01 \x01(\tBQ\n\x13tech.ydb.proto.authZ7github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Auth\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_auth_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023tech.ydb.proto.authZ7github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Auth\370\001\001'
  _LOGINREQUEST._serialized_start=63
  _LOGINREQUEST._serialized_end=168
  _LOGINRESPONSE._serialized_start=170
  _LOGINRESPONSE._serialized_end=231
  _LOGINRESULT._serialized_start=233
  _LOGINRESULT._serialized_end=261
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_issue_message.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eprotos/ydb_issue_message.proto\x12\tYdb.Issue\"\x91\x02\n\x0cIssueMessage\x12\x32\n\x08position\x18\x01 \x01(\x0b\x32 .Ydb.Issue.IssueMessage.Position\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x36\n\x0c\x65nd_position\x18\x03 \x01(\x0b\x32 .Ydb.Issue.IssueMessage.Position\x12\x12\n\nissue_code\x18\x04 \x01(\r\x12\x10\n\x08severity\x18\x05 \x01(\r\x12\'\n\x06issues\x18\x06 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x1a\x35\n\x08Position\x12\x0b\n\x03row\x18\x01 \x01(\r\x12\x0e\n\x06\x63olumn\x18\x02 \x01(\r\x12\x0c\n\x04\x66ile\x18\x03 \x01(\tBM\n\x0etech.ydb.protoZ8github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Issue\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_issue_message_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\016tech.ydb.protoZ8github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Issue\370\001\001'
  _ISSUEMESSAGE._serialized_start=46
  _ISSUEMESSAGE._serialized_end=319
  _ISSUEMESSAGE_POSITION._serialized_start=266
  _ISSUEMESSAGE_POSITION._serialized_end=319
# @@protoc_insertion_point(module_scope)



import pytest
import unittest.mock
import ydb
import asyncio
from ydb import _apis


TEST_ERROR = "Test error"
TEST_QUERY = "SELECT 1 + 2 AS sum"


@pytest.fixture
def mock_connection():
    """Mock a YDB connection to avoid actual connections."""
    with unittest.mock.patch("ydb.connection.Connection.ready_factory") as mock_factory:
        # Setup the mock to return a connection-like object
        mock_connection = unittest.mock.MagicMock()
        # Use the endpoint fixture value via the function parameter
        mock_connection.endpoint = "localhost:2136"  # Will be overridden in tests
        mock_connection.node_id = "mock_node_id"
        mock_factory.return_value = mock_connection
        yield mock_factory


@pytest.fixture
def mock_aio_connection():
    """Mock a YDB async connection to avoid actual connections."""
    with unittest.mock.patch("ydb.aio.connection.Connection.__init__") as mock_init:
        # Setup the mock to return None (as __init__ does)
        mock_init.return_value = None

        # Mock connection_ready method
        with unittest.mock.patch("ydb.aio.connection.Connection.connection_ready") as mock_ready:
            # Create event loop if there isn't one currently
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)

            future = asyncio.Future()
            future.set_result(None)
            mock_ready.return_value = future
            yield mock_init


def create_mock_discovery_resolver(path):
    """Create a mock discovery resolver that raises exception if called."""

    def _mock_fixture():
        with unittest.mock.patch(path) as mock_resolve:
            # Configure mock to throw an exception if called
            mock_resolve.side_effect = Exception("Discovery should not be executed when discovery is disabled")
            yield mock_resolve

    return _mock_fixture


# Mock discovery resolvers to verify no discovery requests are made
mock_discovery_resolver = pytest.fixture(
    create_mock_discovery_resolver("ydb.resolver.DiscoveryEndpointsResolver.context_resolve")
)
mock_aio_discovery_resolver = pytest.fixture(
    create_mock_discovery_resolver("ydb.aio.resolver.DiscoveryEndpointsResolver.resolve")
)


# Basic unit tests for DriverConfig
def test_driver_config_has_disable_discovery_option(endpoint, database):
    """Test that DriverConfig has the disable_discovery option."""
    config = ydb.DriverConfig(endpoint=endpoint, database=database, disable_discovery=True)
    assert hasattr(config, "disable_discovery")
    assert config.disable_discovery is True


# Driver config fixtures
def create_driver_config(endpoint, database, disable_discovery):
    """Create a driver config with the given discovery setting."""
    return ydb.DriverConfig(
        endpoint=endpoint,
        database=database,
        disable_discovery=disable_discovery,
    )


@pytest.fixture
def driver_config_disabled_discovery(endpoint, database):
    """A driver config with discovery disabled"""
    return create_driver_config(endpoint, database, True)


@pytest.fixture
def driver_config_enabled_discovery(endpoint, database):
    """A driver config with discovery enabled (default)"""
    return create_driver_config(endpoint, database, False)


# Common test assertions
def assert_discovery_disabled(driver):
    """Assert that discovery is disabled in the driver."""
    assert "Discovery is disabled" in driver.discovery_debug_details()


def create_future_with_error():
    """Create a future with a test error."""
    future = asyncio.Future()
    future.set_exception(ydb.issues.Error(TEST_ERROR))
    return future


def create_completed_future():
    """Create a completed future."""
    future = asyncio.Future()
    future.set_result(None)
    return future


# Mock tests for synchronous driver
def test_sync_driver_discovery_disabled_mock(
    driver_config_disabled_discovery, mock_connection, mock_discovery_resolver
):
    """Test that when disable_discovery=True, the discovery thread is not started and resolver is not called (mock)."""
    with unittest.mock.patch("ydb.pool.Discovery") as mock_discovery_class:
        driver = ydb.Driver(driver_config=driver_config_disabled_discovery)

        try:
            # Check that the discovery thread was not created
            mock_discovery_class.assert_not_called()

            # Check that discovery is disabled in debug details
            assert_discovery_disabled(driver)

            # Execute a dummy call to verify no discovery requests are made
            try:
                driver(ydb.issues.Error(TEST_ERROR), _apis.OperationService.Stub, "GetOperation")
            except ydb.issues.Error:
                pass  # Expected exception, we just want to ensure no discovery occurs

            # Verify the mock wasn't called
            assert (
                not mock_discovery_resolver.called
            ), "Discovery resolver should not be called when discovery is disabled"
        finally:
            # Clean up
            driver.stop()


def test_sync_driver_discovery_enabled_mock(driver_config_enabled_discovery, mock_connection):
    """Test that when disable_discovery=False, the discovery thread is started (mock)."""
    with unittest.mock.patch("ydb.pool.Discovery") as mock_discovery_class:
        mock_discovery_instance = unittest.mock.MagicMock()
        mock_discovery_class.return_value = mock_discovery_instance

        driver = ydb.Driver(driver_config=driver_config_enabled_discovery)

        try:
            # Check that the discovery thread was created and started
            mock_discovery_class.assert_called_once()
            assert mock_discovery_instance.start.called
        finally:
            # Clean up
            driver.stop()


# Helper for setting up async driver test mocks
def setup_async_driver_mocks():
    """Set up common mocks for async driver tests."""
    mocks = {}

    # Create mock for Discovery class
    discovery_patcher = unittest.mock.patch("ydb.aio.pool.Discovery")
    mocks["mock_discovery_class"] = discovery_patcher.start()

    # Mock the event loop
    loop_patcher = unittest.mock.patch("asyncio.get_event_loop")
    mock_loop = loop_patcher.start()
    mock_loop_instance = unittest.mock.MagicMock()
    mock_loop.return_value = mock_loop_instance
    mock_loop_instance.create_task.return_value = unittest.mock.MagicMock()
    mocks["mock_loop"] = mock_loop

    # Mock the connection pool stop method
    stop_patcher = unittest.mock.patch("ydb.aio.pool.ConnectionPool.stop")
    mock_stop = stop_patcher.start()
    mock_stop.return_value = create_completed_future()
    mocks["mock_stop"] = mock_stop

    # Add cleanup for all patchers
    mocks["patchers"] = [discovery_patcher, loop_patcher, stop_patcher]

    return mocks


def teardown_async_mocks(mocks):
    """Clean up all mock patchers."""
    for patcher in mocks["patchers"]:
        patcher.stop()


# Mock tests for asynchronous driver
@pytest.mark.asyncio
async def test_aio_driver_discovery_disabled_mock(
    driver_config_disabled_discovery, mock_aio_connection, mock_aio_discovery_resolver
):
    """Test that when disable_discovery=True, the discovery is not created and resolver is not called (mock)."""
    mocks = setup_async_driver_mocks()

    try:
        # Mock the pool's call method to prevent unhandled exceptions
        with unittest.mock.patch("ydb.aio.pool.ConnectionPool.__call__") as mock_call:
            mock_call.return_value = create_future_with_error()

            driver = ydb.aio.Driver(driver_config=driver_config_disabled_discovery)

            try:
                # Check that the discovery class was not instantiated
                mocks["mock_discovery_class"].assert_not_called()

                # Check that discovery is disabled in debug details
                assert_discovery_disabled(driver)

                # Execute a dummy call to verify no discovery requests are made
                try:
                    try:
                        await driver(ydb.issues.Error(TEST_ERROR), _apis.OperationService.Stub, "GetOperation")
                    except ydb.issues.Error:
                        pass  # Expected exception, we just want to ensure no discovery occurs
                except Exception as e:
                    if "discovery is disabled" in str(e).lower():
                        raise  # If the error is related to discovery being disabled, re-raise it
                    pass  # Other exceptions are expected as we're using mocks

                # Verify the mock wasn't called
                assert (
                    not mock_aio_discovery_resolver.called
                ), "Discovery resolver should not be called when discovery is disabled"
            finally:
                # The stop method is already mocked, so we don't need to await it
                pass
    finally:
        teardown_async_mocks(mocks)


@pytest.mark.asyncio
async def test_aio_driver_discovery_enabled_mock(driver_config_enabled_discovery, mock_aio_connection):
    """Test that when disable_discovery=False, the discovery is created (mock)."""
    mocks = setup_async_driver_mocks()

    try:
        mock_discovery_instance = unittest.mock.MagicMock()
        mocks["mock_discovery_class"].return_value = mock_discovery_instance

        driver = ydb.aio.Driver(driver_config=driver_config_enabled_discovery)

        try:
            # Check that the discovery class was instantiated
            mocks["mock_discovery_class"].assert_called_once()
            assert driver is not None  # Use the driver variable to avoid F841
        finally:
            # The stop method is already mocked, so we don't need to await it
            pass
    finally:
        teardown_async_mocks(mocks)


# Common integration test logic
def perform_integration_test_checks(driver, is_async=False):
    """Common assertions for integration tests."""
    assert_discovery_disabled(driver)


# Integration tests with real YDB
def test_integration_disable_discovery(driver_config_disabled_discovery):
    """Integration test that tests the disable_discovery feature with a real YDB container."""
    # Create driver with discovery disabled
    driver = ydb.Driver(driver_config=driver_config_disabled_discovery)
    try:
        driver.wait(timeout=15)
        perform_integration_test_checks(driver)

        # Try to execute a simple query to ensure it works with discovery disabled
        with ydb.SessionPool(driver) as pool:

            def query_callback(session):
                result_sets = session.transaction().execute(TEST_QUERY, commit_tx=True)
                assert len(result_sets) == 1
                assert result_sets[0].rows[0].sum == 3

            pool.retry_operation_sync(query_callback)
    finally:
        driver.stop(timeout=10)


@pytest.mark.asyncio
async def test_integration_aio_disable_discovery(driver_config_disabled_discovery):
    """Integration test that tests the disable_discovery feature with a real YDB container (async)."""
    # Create driver with discovery disabled
    driver = ydb.aio.Driver(driver_config=driver_config_disabled_discovery)
    try:
        await driver.wait(timeout=15)
        perform_integration_test_checks(driver, is_async=True)

        # Try to execute a simple query to ensure it works with discovery disabled
        session_pool = ydb.aio.SessionPool(driver, size=10)

        async def query_callback(session):
            result_sets = await session.transaction().execute(TEST_QUERY, commit_tx=True)
            assert len(result_sets) == 1
            assert result_sets[0].rows[0].sum == 3

        try:
            await session_pool.retry_operation(query_callback)
        finally:
            await session_pool.stop()
    finally:
        await driver.stop(timeout=10)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: draft/protos/ydb_keyvalue.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'draft/protos/ydb_keyvalue.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v6.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x64raft/protos/ydb_keyvalue.proto\x12\x0cYdb.KeyValue\x1a\x1aprotos/ydb_operation.proto\"\xf0\x01\n\x12StorageChannelInfo\x12\x17\n\x0fstorage_channel\x18\x01 \x01(\r\x12@\n\x0bstatus_flag\x18\x02 \x01(\x0e\x32+.Ydb.KeyValue.StorageChannelInfo.StatusFlag\"\x7f\n\nStatusFlag\x12\x1b\n\x17STATUS_FLAG_UNSPECIFIED\x10\x00\x12\x15\n\x11STATUS_FLAG_GREEN\x10\n\x12\x1b\n\x17STATUS_FLAG_YELLOW_STOP\x10\x14\x12 \n\x1cSTATUS_FLAG_ORANGE_OUT_SPACE\x10\x1e\"b\n\nPriorities\"T\n\x08Priority\x12\x18\n\x14PRIORITY_UNSPECIFIED\x10\x00\x12\x15\n\x11PRIORITY_REALTIME\x10\x01\x12\x17\n\x13PRIORITY_BACKGROUND\x10\x02\"k\n\rStorageConfig\x12:\n\x07\x63hannel\x18\x01 \x03(\x0b\x32).Ydb.KeyValue.StorageConfig.ChannelConfig\x1a\x1e\n\rChannelConfig\x12\r\n\x05media\x18\x01 \x01(\t\"\x98\x01\n\x08KeyRange\x12\x1c\n\x12\x66rom_key_inclusive\x18\x01 \x01(\tH\x00\x12\x1c\n\x12\x66rom_key_exclusive\x18\x02 \x01(\tH\x00\x12\x1a\n\x10to_key_inclusive\x18\x03 \x01(\tH\x01\x12\x1a\n\x10to_key_exclusive\x18\x04 \x01(\tH\x01\x42\x0c\n\nfrom_boundB\n\n\x08to_bound\"s\n\x12\x41\x63quireLockRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x04\"C\n\x13\x41\x63quireLockResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"=\n\x11\x41\x63quireLockResult\x12\x17\n\x0flock_generation\x18\x01 \x01(\x04\x12\x0f\n\x07node_id\x18\x02 \x01(\r\"\xac\t\n\x19\x45xecuteTransactionRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x04\x12\x1c\n\x0flock_generation\x18\x04 \x01(\x04H\x00\x88\x01\x01\x12\x41\n\x08\x63ommands\x18\x05 \x03(\x0b\x32/.Ydb.KeyValue.ExecuteTransactionRequest.Command\x1a\xba\x07\n\x07\x43ommand\x12S\n\x0c\x64\x65lete_range\x18\x01 \x01(\x0b\x32;.Ydb.KeyValue.ExecuteTransactionRequest.Command.DeleteRangeH\x00\x12H\n\x06rename\x18\x02 \x01(\x0b\x32\x36.Ydb.KeyValue.ExecuteTransactionRequest.Command.RenameH\x00\x12O\n\ncopy_range\x18\x03 \x01(\x0b\x32\x39.Ydb.KeyValue.ExecuteTransactionRequest.Command.CopyRangeH\x00\x12H\n\x06\x63oncat\x18\x04 \x01(\x0b\x32\x36.Ydb.KeyValue.ExecuteTransactionRequest.Command.ConcatH\x00\x12\x46\n\x05write\x18\x05 \x01(\x0b\x32\x35.Ydb.KeyValue.ExecuteTransactionRequest.Command.WriteH\x00\x1a*\n\x06Rename\x12\x0f\n\x07old_key\x18\x01 \x01(\t\x12\x0f\n\x07new_key\x18\x02 \x01(\t\x1a\x45\n\x06\x43oncat\x12\x12\n\ninput_keys\x18\x01 \x03(\t\x12\x12\n\noutput_key\x18\x02 \x01(\t\x12\x13\n\x0bkeep_inputs\x18\x03 \x01(\x08\x1a\x63\n\tCopyRange\x12%\n\x05range\x18\x01 \x01(\x0b\x32\x16.Ydb.KeyValue.KeyRange\x12\x18\n\x10prefix_to_remove\x18\x02 \x01(\t\x12\x15\n\rprefix_to_add\x18\x03 \x01(\t\x1a\x94\x02\n\x05Write\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x17\n\x0fstorage_channel\x18\x03 \x01(\r\x12\x33\n\x08priority\x18\x04 \x01(\x0e\x32!.Ydb.KeyValue.Priorities.Priority\x12L\n\x06tactic\x18\x05 \x01(\x0e\x32<.Ydb.KeyValue.ExecuteTransactionRequest.Command.Write.Tactic\"S\n\x06Tactic\x12\x16\n\x12TACTIC_UNSPECIFIED\x10\x00\x12\x19\n\x15TACTIC_MAX_THROUGHPUT\x10\x01\x12\x16\n\x12TACTIC_MIN_LATENCY\x10\x02\x1a\x34\n\x0b\x44\x65leteRange\x12%\n\x05range\x18\x01 \x01(\x0b\x32\x16.Ydb.KeyValue.KeyRangeB\x08\n\x06\x61\x63tionB\x12\n\x10_lock_generation\"J\n\x1a\x45xecuteTransactionResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"k\n\x18\x45xecuteTransactionResult\x12>\n\x14storage_channel_info\x18\x01 \x03(\x0b\x32 .Ydb.KeyValue.StorageChannelInfo\x12\x0f\n\x07node_id\x18\x02 \x01(\r\"\x93\x02\n\x0bReadRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x04\x12\x1c\n\x0flock_generation\x18\x04 \x01(\x04H\x00\x88\x01\x01\x12\x0b\n\x03key\x18\x05 \x01(\t\x12\x0e\n\x06offset\x18\x06 \x01(\x04\x12\x0c\n\x04size\x18\x07 \x01(\x04\x12\x13\n\x0blimit_bytes\x18\x08 \x01(\x04\x12\x33\n\x08priority\x18\t \x01(\x0e\x32!.Ydb.KeyValue.Priorities.PriorityB\x12\n\x10_lock_generation\"<\n\x0cReadResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x89\x01\n\nReadResult\x12\x15\n\rrequested_key\x18\x01 \x01(\t\x12\x18\n\x10requested_offset\x18\x02 \x01(\x04\x12\x16\n\x0erequested_size\x18\x03 \x01(\x04\x12\r\n\x05value\x18\x04 \x01(\x0c\x12\x12\n\nis_overrun\x18\x05 \x01(\x08\x12\x0f\n\x07node_id\x18\x06 \x01(\r\"\x94\x02\n\x10ReadRangeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x04\x12\x1c\n\x0flock_generation\x18\x04 \x01(\x04H\x00\x88\x01\x01\x12%\n\x05range\x18\x05 \x01(\x0b\x32\x16.Ydb.KeyValue.KeyRange\x12\x13\n\x0blimit_bytes\x18\x06 \x01(\x04\x12\x33\n\x08priority\x18\x07 \x01(\x0e\x32!.Ydb.KeyValue.Priorities.PriorityB\x12\n\x10_lock_generation\"A\n\x11ReadRangeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xd1\x01\n\x0fReadRangeResult\x12\x38\n\x04pair\x18\x01 \x03(\x0b\x32*.Ydb.KeyValue.ReadRangeResult.KeyValuePair\x12\x12\n\nis_overrun\x18\x02 \x01(\x08\x12\x0f\n\x07node_id\x18\x03 \x01(\r\x1a_\n\x0cKeyValuePair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x1a\n\x12\x63reation_unix_time\x18\x04 \x01(\x04\x12\x17\n\x0fstorage_channel\x18\x05 \x01(\r\"\xdf\x01\n\x10ListRangeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x04\x12\x1c\n\x0flock_generation\x18\x04 \x01(\x04H\x00\x88\x01\x01\x12%\n\x05range\x18\x05 \x01(\x0b\x32\x16.Ydb.KeyValue.KeyRange\x12\x13\n\x0blimit_bytes\x18\x06 \x01(\x04\x42\x12\n\x10_lock_generation\"A\n\x11ListRangeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xcb\x01\n\x0fListRangeResult\x12\x32\n\x03key\x18\x01 \x03(\x0b\x32%.Ydb.KeyValue.ListRangeResult.KeyInfo\x12\x12\n\nis_overrun\x18\x02 \x01(\x08\x12\x0f\n\x07node_id\x18\x03 \x01(\r\x1a_\n\x07KeyInfo\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x12\n\nvalue_size\x18\x02 \x01(\r\x12\x1a\n\x12\x63reation_unix_time\x18\x03 \x01(\x04\x12\x17\n\x0fstorage_channel\x18\x04 \x01(\r\"\xca\x01\n\x1eGetStorageChannelStatusRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x04\x12\x1c\n\x0flock_generation\x18\x04 \x01(\x04H\x00\x88\x01\x01\x12\x17\n\x0fstorage_channel\x18\x05 \x03(\rB\x12\n\x10_lock_generation\"O\n\x1fGetStorageChannelStatusResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"p\n\x1dGetStorageChannelStatusResult\x12>\n\x14storage_channel_info\x18\x01 \x03(\x0b\x32 .Ydb.KeyValue.StorageChannelInfo\x12\x0f\n\x07node_id\x18\x02 \x01(\r\"\xac\x01\n\x13\x43reateVolumeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x17\n\x0fpartition_count\x18\x04 \x01(\r\x12\x33\n\x0estorage_config\x18\x05 \x01(\x0b\x32\x1b.Ydb.KeyValue.StorageConfig\"D\n\x14\x43reateVolumeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x14\n\x12\x43reateVolumeResult\"\\\n\x11\x44ropVolumeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"B\n\x12\x44ropVolumeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x12\n\x10\x44ropVolumeResult\"\xb1\x01\n\x12\x41lterVolumeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x1d\n\x15\x61lter_partition_count\x18\x03 \x01(\r\x12\x33\n\x0estorage_config\x18\x04 \x01(\x0b\x32\x1b.Ydb.KeyValue.StorageConfig\"C\n\x13\x41lterVolumeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x13\n\x11\x41lterVolumeResult\"`\n\x15\x44\x65scribeVolumeRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"F\n\x16\x44\x65scribeVolumeResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"=\n\x14\x44\x65scribeVolumeResult\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x17\n\x0fpartition_count\x18\x02 \x01(\x04\"v\n\x1aListLocalPartitionsRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x0f\n\x07node_id\x18\x03 \x01(\x04\"K\n\x1bListLocalPartitionsResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"Q\n\x19ListLocalPartitionsResult\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0f\n\x07node_id\x18\x02 \x01(\x04\x12\x15\n\rpartition_ids\x18\x03 \x03(\x04\x42h\n tech.ydb.proto.draft.keyvalue.v1ZAgithub.com/ydb-platform/ydb-go-genproto/draft/protos/Ydb_KeyValue\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'draft.protos.ydb_keyvalue_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n tech.ydb.proto.draft.keyvalue.v1ZAgithub.com/ydb-platform/ydb-go-genproto/draft/protos/Ydb_KeyValue\370\001\001'
  _globals['_STORAGECHANNELINFO']._serialized_start=78
  _globals['_STORAGECHANNELINFO']._serialized_end=318
  _globals['_STORAGECHANNELINFO_STATUSFLAG']._serialized_start=191
  _globals['_STORAGECHANNELINFO_STATUSFLAG']._serialized_end=318
  _globals['_PRIORITIES']._serialized_start=320
  _globals['_PRIORITIES']._serialized_end=418
  _globals['_PRIORITIES_PRIORITY']._serialized_start=334
  _globals['_PRIORITIES_PRIORITY']._serialized_end=418
  _globals['_STORAGECONFIG']._serialized_start=420
  _globals['_STORAGECONFIG']._serialized_end=527
  _globals['_STORAGECONFIG_CHANNELCONFIG']._serialized_start=497
  _globals['_STORAGECONFIG_CHANNELCONFIG']._serialized_end=527
  _globals['_KEYRANGE']._serialized_start=530
  _globals['_KEYRANGE']._serialized_end=682
  _globals['_ACQUIRELOCKREQUEST']._serialized_start=684
  _globals['_ACQUIRELOCKREQUEST']._serialized_end=799
  _globals['_ACQUIRELOCKRESPONSE']._serialized_start=801
  _globals['_ACQUIRELOCKRESPONSE']._serialized_end=868
  _globals['_ACQUIRELOCKRESULT']._serialized_start=870
  _globals['_ACQUIRELOCKRESULT']._serialized_end=931
  _globals['_EXECUTETRANSACTIONREQUEST']._serialized_start=934
  _globals['_EXECUTETRANSACTIONREQUEST']._serialized_end=2130
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND']._serialized_start=1156
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND']._serialized_end=2110
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_RENAME']._serialized_start=1553
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_RENAME']._serialized_end=1595
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_CONCAT']._serialized_start=1597
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_CONCAT']._serialized_end=1666
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_COPYRANGE']._serialized_start=1668
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_COPYRANGE']._serialized_end=1767
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_WRITE']._serialized_start=1770
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_WRITE']._serialized_end=2046
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_WRITE_TACTIC']._serialized_start=1963
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_WRITE_TACTIC']._serialized_end=2046
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_DELETERANGE']._serialized_start=2048
  _globals['_EXECUTETRANSACTIONREQUEST_COMMAND_DELETERANGE']._serialized_end=2100
  _globals['_EXECUTETRANSACTIONRESPONSE']._serialized_start=2132
  _globals['_EXECUTETRANSACTIONRESPONSE']._serialized_end=2206
  _globals['_EXECUTETRANSACTIONRESULT']._serialized_start=2208
  _globals['_EXECUTETRANSACTIONRESULT']._serialized_end=2315
  _globals['_READREQUEST']._serialized_start=2318
  _globals['_READREQUEST']._serialized_end=2593
  _globals['_READRESPONSE']._serialized_start=2595
  _globals['_READRESPONSE']._serialized_end=2655
  _globals['_READRESULT']._serialized_start=2658
  _globals['_READRESULT']._serialized_end=2795
  _globals['_READRANGEREQUEST']._serialized_start=2798
  _globals['_READRANGEREQUEST']._serialized_end=3074
  _globals['_READRANGERESPONSE']._serialized_start=3076
  _globals['_READRANGERESPONSE']._serialized_end=3141
  _globals['_READRANGERESULT']._serialized_start=3144
  _globals['_READRANGERESULT']._serialized_end=3353
  _globals['_READRANGERESULT_KEYVALUEPAIR']._serialized_start=3258
  _globals['_READRANGERESULT_KEYVALUEPAIR']._serialized_end=3353
  _globals['_LISTRANGEREQUEST']._serialized_start=3356
  _globals['_LISTRANGEREQUEST']._serialized_end=3579
  _globals['_LISTRANGERESPONSE']._serialized_start=3581
  _globals['_LISTRANGERESPONSE']._serialized_end=3646
  _globals['_LISTRANGERESULT']._serialized_start=3649
  _globals['_LISTRANGERESULT']._serialized_end=3852
  _globals['_LISTRANGERESULT_KEYINFO']._serialized_start=3757
  _globals['_LISTRANGERESULT_KEYINFO']._serialized_end=3852
  _globals['_GETSTORAGECHANNELSTATUSREQUEST']._serialized_start=3855
  _globals['_GETSTORAGECHANNELSTATUSREQUEST']._serialized_end=4057
  _globals['_GETSTORAGECHANNELSTATUSRESPONSE']._serialized_start=4059
  _globals['_GETSTORAGECHANNELSTATUSRESPONSE']._serialized_end=4138
  _globals['_GETSTORAGECHANNELSTATUSRESULT']._serialized_start=4140
  _globals['_GETSTORAGECHANNELSTATUSRESULT']._serialized_end=4252
  _globals['_CREATEVOLUMEREQUEST']._serialized_start=4255
  _globals['_CREATEVOLUMEREQUEST']._serialized_end=4427
  _globals['_CREATEVOLUMERESPONSE']._serialized_start=4429
  _globals['_CREATEVOLUMERESPONSE']._serialized_end=4497
  _globals['_CREATEVOLUMERESULT']._serialized_start=4499
  _globals['_CREATEVOLUMERESULT']._serialized_end=4519
  _globals['_DROPVOLUMEREQUEST']._serialized_start=4521
  _globals['_DROPVOLUMEREQUEST']._serialized_end=4613
  _globals['_DROPVOLUMERESPONSE']._serialized_start=4615
  _globals['_DROPVOLUMERESPONSE']._serialized_end=4681
  _globals['_DROPVOLUMERESULT']._serialized_start=4683
  _globals['_DROPVOLUMERESULT']._serialized_end=4701
  _globals['_ALTERVOLUMEREQUEST']._serialized_start=4704
  _globals['_ALTERVOLUMEREQUEST']._serialized_end=4881
  _globals['_ALTERVOLUMERESPONSE']._serialized_start=4883
  _globals['_ALTERVOLUMERESPONSE']._serialized_end=4950
  _globals['_ALTERVOLUMERESULT']._serialized_start=4952
  _globals['_ALTERVOLUMERESULT']._serialized_end=4971
  _globals['_DESCRIBEVOLUMEREQUEST']._serialized_start=4973
  _globals['_DESCRIBEVOLUMEREQUEST']._serialized_end=5069
  _globals['_DESCRIBEVOLUMERESPONSE']._serialized_start=5071
  _globals['_DESCRIBEVOLUMERESPONSE']._serialized_end=5141
  _globals['_DESCRIBEVOLUMERESULT']._serialized_start=5143
  _globals['_DESCRIBEVOLUMERESULT']._serialized_end=5204
  _globals['_LISTLOCALPARTITIONSREQUEST']._serialized_start=5206
  _globals['_LISTLOCALPARTITIONSREQUEST']._serialized_end=5324
  _globals['_LISTLOCALPARTITIONSRESPONSE']._serialized_start=5326
  _globals['_LISTLOCALPARTITIONSRESPONSE']._serialized_end=5401
  _globals['_LISTLOCALPARTITIONSRESULT']._serialized_start=5403
  _globals['_LISTLOCALPARTITIONSRESULT']._serialized_end=5484
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_topic.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v4.protos import ydb_operation_pb2 as protos_dot_ydb__operation__pb2
from ydb._grpc.v4.protos import ydb_scheme_pb2 as protos_dot_ydb__scheme__pb2
from ydb._grpc.v4.protos import ydb_status_codes_pb2 as protos_dot_ydb__status__codes__pb2
from ydb._grpc.v4.protos import ydb_issue_message_pb2 as protos_dot_ydb__issue__message__pb2
from ydb._grpc.v4.protos.annotations import sensitive_pb2 as protos_dot_annotations_dot_sensitive__pb2
from ydb._grpc.v4.protos.annotations import validation_pb2 as protos_dot_annotations_dot_validation__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16protos/ydb_topic.proto\x12\tYdb.Topic\x1a\x1aprotos/ydb_operation.proto\x1a\x17protos/ydb_scheme.proto\x1a\x1dprotos/ydb_status_codes.proto\x1a\x1eprotos/ydb_issue_message.proto\x1a\"protos/annotations/sensitive.proto\x1a#protos/annotations/validation.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"7\n\x0fSupportedCodecs\x12$\n\x06\x63odecs\x18\x01 \x03(\x05\x42\x14\xb2\xe6*\n[1; 19999]\x9a\xe6*\x02\x18\x64\"*\n\x0cOffsetsRange\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x03\")\n\x12UpdateTokenRequest\x12\x13\n\x05token\x18\x01 \x01(\tB\x04\xb8\xe6*\x01\"\x15\n\x13UpdateTokenResponse\"C\n\x17PartitionWithGeneration\x12\x14\n\x0cpartition_id\x18\x01 \x01(\x03\x12\x12\n\ngeneration\x18\x02 \x01(\x03\"*\n\x0cMetadataItem\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\"\x9e\x14\n\x12StreamWriteMessage\x1a\xe5\x01\n\nFromClient\x12\x41\n\x0cinit_request\x18\x01 \x01(\x0b\x32).Ydb.Topic.StreamWriteMessage.InitRequestH\x00\x12\x43\n\rwrite_request\x18\x02 \x01(\x0b\x32*.Ydb.Topic.StreamWriteMessage.WriteRequestH\x00\x12=\n\x14update_token_request\x18\x03 \x01(\x0b\x32\x1d.Ydb.Topic.UpdateTokenRequestH\x00\x42\x10\n\x0e\x63lient_message\x1a\xbf\x02\n\nFromServer\x12)\n\x06status\x18\x01 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\x12\'\n\x06issues\x18\x02 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x12\x43\n\rinit_response\x18\x03 \x01(\x0b\x32*.Ydb.Topic.StreamWriteMessage.InitResponseH\x00\x12\x45\n\x0ewrite_response\x18\x04 \x01(\x0b\x32+.Ydb.Topic.StreamWriteMessage.WriteResponseH\x00\x12?\n\x15update_token_response\x18\x05 \x01(\x0b\x32\x1e.Ydb.Topic.UpdateTokenResponseH\x00\x42\x10\n\x0eserver_message\x1a\xfe\x02\n\x0bInitRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x1c\n\x0bproducer_id\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x10\x12[\n\x12write_session_meta\x18\x03 \x03(\x0b\x32?.Ydb.Topic.StreamWriteMessage.InitRequest.WriteSessionMetaEntry\x12#\n\x10message_group_id\x18\x04 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x10H\x00\x12\x16\n\x0cpartition_id\x18\x05 \x01(\x03H\x00\x12G\n\x19partition_with_generation\x18\x07 \x01(\x0b\x32\".Ydb.Topic.PartitionWithGenerationH\x00\x12\x17\n\x0fget_last_seq_no\x18\x06 \x01(\x08\x1a\x37\n\x15WriteSessionMetaEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0e\n\x0cpartitioning\x1a\x83\x01\n\x0cInitResponse\x12\x13\n\x0blast_seq_no\x18\x01 \x01(\x03\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x03\x12\x34\n\x10supported_codecs\x18\x04 \x01(\x0b\x32\x1a.Ydb.Topic.SupportedCodecs\x1a\xe8\x03\n\x0cWriteRequest\x12H\n\x08messages\x18\x01 \x03(\x0b\x32\x36.Ydb.Topic.StreamWriteMessage.WriteRequest.MessageData\x12\r\n\x05\x63odec\x18\x02 \x01(\x05\x12/\n\x02tx\x18\x03 \x01(\x0b\x32\x1e.Ydb.Topic.TransactionIdentityH\x00\x88\x01\x01\x1a\xc6\x02\n\x0bMessageData\x12\x0e\n\x06seq_no\x18\x01 \x01(\x03\x12.\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x19\n\x11uncompressed_size\x18\x04 \x01(\x03\x12#\n\x10message_group_id\x18\x05 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x10H\x00\x12\x16\n\x0cpartition_id\x18\x06 \x01(\x03H\x00\x12G\n\x19partition_with_generation\x18\x08 \x01(\x0b\x32\".Ydb.Topic.PartitionWithGenerationH\x00\x12\x38\n\x0emetadata_items\x18\x07 \x03(\x0b\x32\x17.Ydb.Topic.MetadataItemB\x07\x9a\xe6*\x03\x18\xe8\x07\x42\x0e\n\x0cpartitioningB\x05\n\x03_tx\x1a\xeb\x07\n\rWriteResponse\x12\x42\n\x04\x61\x63ks\x18\x01 \x03(\x0b\x32\x34.Ydb.Topic.StreamWriteMessage.WriteResponse.WriteAck\x12\x14\n\x0cpartition_id\x18\x02 \x01(\x03\x12U\n\x10write_statistics\x18\x03 \x01(\x0b\x32;.Ydb.Topic.StreamWriteMessage.WriteResponse.WriteStatistics\x1a\xf8\x03\n\x08WriteAck\x12\x0e\n\x06seq_no\x18\x01 \x01(\x03\x12O\n\x07written\x18\x02 \x01(\x0b\x32<.Ydb.Topic.StreamWriteMessage.WriteResponse.WriteAck.WrittenH\x00\x12O\n\x07skipped\x18\x03 \x01(\x0b\x32<.Ydb.Topic.StreamWriteMessage.WriteResponse.WriteAck.SkippedH\x00\x12Y\n\rwritten_in_tx\x18\x04 \x01(\x0b\x32@.Ydb.Topic.StreamWriteMessage.WriteResponse.WriteAck.WrittenInTxH\x00\x1a\x19\n\x07Written\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x1a\x9c\x01\n\x07Skipped\x12S\n\x06reason\x18\x01 \x01(\x0e\x32\x43.Ydb.Topic.StreamWriteMessage.WriteResponse.WriteAck.Skipped.Reason\"<\n\x06Reason\x12\x16\n\x12REASON_UNSPECIFIED\x10\x00\x12\x1a\n\x16REASON_ALREADY_WRITTEN\x10\x01\x1a\r\n\x0bWrittenInTxB\x16\n\x14message_write_status\x1a\xad\x02\n\x0fWriteStatistics\x12\x32\n\x0fpersisting_time\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x36\n\x13min_queue_wait_time\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x36\n\x13max_queue_wait_time\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12<\n\x19partition_quota_wait_time\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x38\n\x15topic_quota_wait_time\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\"\x9c#\n\x11StreamReadMessage\x1aT\n\x10PartitionSession\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x03\x1a\xb1\x05\n\nFromClient\x12@\n\x0cinit_request\x18\x01 \x01(\x0b\x32(.Ydb.Topic.StreamReadMessage.InitRequestH\x00\x12@\n\x0cread_request\x18\x02 \x01(\x0b\x32(.Ydb.Topic.StreamReadMessage.ReadRequestH\x00\x12Q\n\x15\x63ommit_offset_request\x18\x03 \x01(\x0b\x32\x30.Ydb.Topic.StreamReadMessage.CommitOffsetRequestH\x00\x12\x66\n partition_session_status_request\x18\x04 \x01(\x0b\x32:.Ydb.Topic.StreamReadMessage.PartitionSessionStatusRequestH\x00\x12=\n\x14update_token_request\x18\x05 \x01(\x0b\x32\x1d.Ydb.Topic.UpdateTokenRequestH\x00\x12\x45\n\x0f\x64irect_read_ack\x18\x08 \x01(\x0b\x32*.Ydb.Topic.StreamReadMessage.DirectReadAckH\x00\x12\x66\n start_partition_session_response\x18\x06 \x01(\x0b\x32:.Ydb.Topic.StreamReadMessage.StartPartitionSessionResponseH\x00\x12\x64\n\x1fstop_partition_session_response\x18\x07 \x01(\x0b\x32\x39.Ydb.Topic.StreamReadMessage.StopPartitionSessionResponseH\x00\x42\x10\n\x0e\x63lient_message\x1a\xf0\x06\n\nFromServer\x12)\n\x06status\x18\x01 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\x12\'\n\x06issues\x18\x02 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x12\x42\n\rinit_response\x18\x03 \x01(\x0b\x32).Ydb.Topic.StreamReadMessage.InitResponseH\x00\x12\x42\n\rread_response\x18\x04 \x01(\x0b\x32).Ydb.Topic.StreamReadMessage.ReadResponseH\x00\x12S\n\x16\x63ommit_offset_response\x18\x05 \x01(\x0b\x32\x31.Ydb.Topic.StreamReadMessage.CommitOffsetResponseH\x00\x12h\n!partition_session_status_response\x18\x06 \x01(\x0b\x32;.Ydb.Topic.StreamReadMessage.PartitionSessionStatusResponseH\x00\x12?\n\x15update_token_response\x18\x07 \x01(\x0b\x32\x1e.Ydb.Topic.UpdateTokenResponseH\x00\x12\x64\n\x1fstart_partition_session_request\x18\x08 \x01(\x0b\x32\x39.Ydb.Topic.StreamReadMessage.StartPartitionSessionRequestH\x00\x12\x62\n\x1estop_partition_session_request\x18\t \x01(\x0b\x32\x38.Ydb.Topic.StreamReadMessage.StopPartitionSessionRequestH\x00\x12W\n\x18update_partition_session\x18\n \x01(\x0b\x32\x33.Ydb.Topic.StreamReadMessage.UpdatePartitionSessionH\x00\x12Q\n\x15\x65nd_partition_session\x18\x0b \x01(\x0b\x32\x30.Ydb.Topic.StreamReadMessage.EndPartitionSessionH\x00\x42\x10\n\x0eserver_message\x1a\xdc\x02\n\x0bInitRequest\x12X\n\x14topics_read_settings\x18\x01 \x03(\x0b\x32:.Ydb.Topic.StreamReadMessage.InitRequest.TopicReadSettings\x12\x10\n\x08\x63onsumer\x18\x02 \x01(\t\x12\x13\n\x0breader_name\x18\x03 \x01(\t\x12\x13\n\x0b\x64irect_read\x18\x04 \x01(\x08\x12!\n\x19\x61uto_partitioning_support\x18\x05 \x01(\x08\x1a\x93\x01\n\x11TopicReadSettings\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x15\n\rpartition_ids\x18\x02 \x03(\x03\x12*\n\x07max_lag\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12-\n\tread_from\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\"\n\x0cInitResponse\x12\x12\n\nsession_id\x18\x01 \x01(\t\x1a!\n\x0bReadRequest\x12\x12\n\nbytes_size\x18\x01 \x01(\x03\x1a\x91\x06\n\x0cReadResponse\x12O\n\x0epartition_data\x18\x01 \x03(\x0b\x32\x37.Ydb.Topic.StreamReadMessage.ReadResponse.PartitionData\x12\x12\n\nbytes_size\x18\x02 \x01(\x03\x1a\xda\x01\n\x0bMessageData\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x0e\n\x06seq_no\x18\x02 \x01(\x03\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x12\x19\n\x11uncompressed_size\x18\x06 \x01(\x03\x12!\n\x10message_group_id\x18\x07 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x10\x12/\n\x0emetadata_items\x18\x08 \x03(\x0b\x32\x17.Ydb.Topic.MetadataItem\x1a\xcd\x02\n\x05\x42\x61tch\x12K\n\x0cmessage_data\x18\x01 \x03(\x0b\x32\x35.Ydb.Topic.StreamReadMessage.ReadResponse.MessageData\x12\x1c\n\x0bproducer_id\x18\x02 \x01(\tB\x07\xa2\xe6*\x03\x18\x80\x10\x12\x61\n\x12write_session_meta\x18\x03 \x03(\x0b\x32\x45.Ydb.Topic.StreamReadMessage.ReadResponse.Batch.WriteSessionMetaEntry\x12\r\n\x05\x63odec\x18\x04 \x01(\x05\x12.\n\nwritten_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\x37\n\x15WriteSessionMetaEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1ao\n\rPartitionData\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12@\n\x07\x62\x61tches\x18\x02 \x03(\x0b\x32/.Ydb.Topic.StreamReadMessage.ReadResponse.Batch\x1a\xd6\x01\n\x13\x43ommitOffsetRequest\x12^\n\x0e\x63ommit_offsets\x18\x01 \x03(\x0b\x32\x46.Ydb.Topic.StreamReadMessage.CommitOffsetRequest.PartitionCommitOffset\x1a_\n\x15PartitionCommitOffset\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12(\n\x07offsets\x18\x02 \x03(\x0b\x32\x17.Ydb.Topic.OffsetsRange\x1a\xdc\x01\n\x14\x43ommitOffsetResponse\x12p\n\x1cpartitions_committed_offsets\x18\x01 \x03(\x0b\x32J.Ydb.Topic.StreamReadMessage.CommitOffsetResponse.PartitionCommittedOffset\x1aR\n\x18PartitionCommittedOffset\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x18\n\x10\x63ommitted_offset\x18\x02 \x01(\x03\x1a=\n\x1dPartitionSessionStatusRequest\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x1a\xcb\x01\n\x1ePartitionSessionStatusResponse\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x32\n\x11partition_offsets\x18\x02 \x01(\x0b\x32\x17.Ydb.Topic.OffsetsRange\x12\x18\n\x10\x63ommitted_offset\x18\x03 \x01(\x03\x12=\n\x19write_time_high_watermark\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1a\xf0\x01\n\x1cStartPartitionSessionRequest\x12H\n\x11partition_session\x18\x01 \x01(\x0b\x32-.Ydb.Topic.StreamReadMessage.PartitionSession\x12\x18\n\x10\x63ommitted_offset\x18\x02 \x01(\x03\x12\x32\n\x11partition_offsets\x18\x03 \x01(\x0b\x32\x17.Ydb.Topic.OffsetsRange\x12\x38\n\x12partition_location\x18\x04 \x01(\x0b\x32\x1c.Ydb.Topic.PartitionLocation\x1a\x95\x01\n\x1dStartPartitionSessionResponse\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x18\n\x0bread_offset\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x1a\n\rcommit_offset\x18\x03 \x01(\x03H\x01\x88\x01\x01\x42\x0e\n\x0c_read_offsetB\x10\n\x0e_commit_offset\x1a\x84\x01\n\x1bStopPartitionSessionRequest\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x10\n\x08graceful\x18\x02 \x01(\x08\x12\x18\n\x10\x63ommitted_offset\x18\x03 \x01(\x03\x12\x1b\n\x13last_direct_read_id\x18\x04 \x01(\x03\x1aN\n\x1cStopPartitionSessionResponse\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x10\n\x08graceful\x18\x02 \x01(\x08\x1ap\n\x16UpdatePartitionSession\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x38\n\x12partition_location\x18\x02 \x01(\x0b\x32\x1c.Ydb.Topic.PartitionLocation\x1a\x45\n\rDirectReadAck\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x16\n\x0e\x64irect_read_id\x18\x02 \x01(\x03\x1ap\n\x13\x45ndPartitionSession\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x1e\n\x16\x61\x64jacent_partition_ids\x18\x02 \x03(\x03\x12\x1b\n\x13\x63hild_partition_ids\x18\x03 \x03(\x03\"\x8b\r\n\x17StreamDirectReadMessage\x1a\xa8\x02\n\nFromClient\x12\x46\n\x0cinit_request\x18\x01 \x01(\x0b\x32..Ydb.Topic.StreamDirectReadMessage.InitRequestH\x00\x12\x80\x01\n+start_direct_read_partition_session_request\x18\x02 \x01(\x0b\x32I.Ydb.Topic.StreamDirectReadMessage.StartDirectReadPartitionSessionRequestH\x00\x12=\n\x14update_token_request\x18\x03 \x01(\x0b\x32\x1d.Ydb.Topic.UpdateTokenRequestH\x00\x42\x10\n\x0e\x63lient_message\x1a\xca\x04\n\nFromServer\x12)\n\x06status\x18\x01 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\x12\'\n\x06issues\x18\x02 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x12H\n\rinit_response\x18\x06 \x01(\x0b\x32/.Ydb.Topic.StreamDirectReadMessage.InitResponseH\x00\x12\x82\x01\n,start_direct_read_partition_session_response\x18\x07 \x01(\x0b\x32J.Ydb.Topic.StreamDirectReadMessage.StartDirectReadPartitionSessionResponseH\x00\x12o\n\"stop_direct_read_partition_session\x18\x03 \x01(\x0b\x32\x41.Ydb.Topic.StreamDirectReadMessage.StopDirectReadPartitionSessionH\x00\x12U\n\x14\x64irect_read_response\x18\x04 \x01(\x0b\x32\x35.Ydb.Topic.StreamDirectReadMessage.DirectReadResponseH\x00\x12?\n\x15update_token_response\x18\x05 \x01(\x0b\x32\x1e.Ydb.Topic.UpdateTokenResponseH\x00\x42\x10\n\x0eserver_message\x1a\xb6\x01\n\x0bInitRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12^\n\x14topics_read_settings\x18\x02 \x03(\x0b\x32@.Ydb.Topic.StreamDirectReadMessage.InitRequest.TopicReadSettings\x12\x10\n\x08\x63onsumer\x18\x03 \x01(\t\x1a!\n\x11TopicReadSettings\x12\x0c\n\x04path\x18\x01 \x01(\t\x1a\x0e\n\x0cInitResponse\x1aw\n&StartDirectReadPartitionSessionRequest\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x1b\n\x13last_direct_read_id\x18\x02 \x01(\x03\x12\x12\n\ngeneration\x18\x03 \x01(\x03\x1a[\n\'StartDirectReadPartitionSessionResponse\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x12\n\ngeneration\x18\x02 \x01(\x03\x1a\xa6\x01\n\x1eStopDirectReadPartitionSession\x12)\n\x06status\x18\x01 \x01(\x0e\x32\x19.Ydb.StatusIds.StatusCode\x12\'\n\x06issues\x18\x02 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x12\x1c\n\x14partition_session_id\x18\x03 \x01(\x03\x12\x12\n\ngeneration\x18\x04 \x01(\x03\x1a\xaf\x01\n\x12\x44irectReadResponse\x12\x1c\n\x14partition_session_id\x18\x01 \x01(\x03\x12\x16\n\x0e\x64irect_read_id\x18\x02 \x01(\x03\x12O\n\x0epartition_data\x18\x03 \x01(\x0b\x32\x37.Ydb.Topic.StreamReadMessage.ReadResponse.PartitionData\x12\x12\n\nbytes_size\x18\x04 \x01(\x03\"2\n\x13TransactionIdentity\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07session\x18\x02 \x01(\t\"\xc4\x03\n!UpdateOffsetsInTransactionRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12*\n\x02tx\x18\x02 \x01(\x0b\x32\x1e.Ydb.Topic.TransactionIdentity\x12I\n\x06topics\x18\x03 \x03(\x0b\x32\x39.Ydb.Topic.UpdateOffsetsInTransactionRequest.TopicOffsets\x12\x10\n\x08\x63onsumer\x18\x04 \x01(\t\x1a\xda\x01\n\x0cTopicOffsets\x12\x0c\n\x04path\x18\x01 \x01(\t\x12^\n\npartitions\x18\x02 \x03(\x0b\x32J.Ydb.Topic.UpdateOffsetsInTransactionRequest.TopicOffsets.PartitionOffsets\x1a\\\n\x10PartitionOffsets\x12\x14\n\x0cpartition_id\x18\x01 \x01(\x03\x12\x32\n\x11partition_offsets\x18\x02 \x03(\x0b\x32\x17.Ydb.Topic.OffsetsRange\"R\n\"UpdateOffsetsInTransactionResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\"\n UpdateOffsetsInTransactionResult\"\xaf\x01\n\x13\x43ommitOffsetRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x03\x12\x10\n\x08\x63onsumer\x18\x04 \x01(\t\x12\x0e\n\x06offset\x18\x05 \x01(\x03\x12\x17\n\x0fread_session_id\x18\x06 \x01(\t\"D\n\x14\x43ommitOffsetResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x14\n\x12\x43ommitOffsetResult\"L\n\x13MultipleWindowsStat\x12\x12\n\nper_minute\x18\x01 \x01(\x03\x12\x10\n\x08per_hour\x18\x02 \x01(\x03\x12\x0f\n\x07per_day\x18\x03 \x01(\x03\"\xc3\x05\n\x08\x43onsumer\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\timportant\x18\x02 \x01(\x08\x12-\n\tread_from\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x10supported_codecs\x18\x05 \x01(\x0b\x32\x1a.Ydb.Topic.SupportedCodecs\x12\x37\n\nattributes\x18\x06 \x03(\x0b\x32#.Ydb.Topic.Consumer.AttributesEntry\x12\x39\n\x0e\x63onsumer_stats\x18\x07 \x01(\x0b\x32!.Ydb.Topic.Consumer.ConsumerStats\x12;\n\x13\x61vailability_period\x18\x08 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00\x88\x01\x01\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xae\x02\n\rConsumerStats\x12\x41\n\x1dmin_partitions_last_read_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x11max_read_time_lag\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x35\n\x12max_write_time_lag\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x39\n\x16max_committed_time_lag\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x32\n\nbytes_read\x18\x04 \x01(\x0b\x32\x1e.Ydb.Topic.MultipleWindowsStatB\x16\n\x14_availability_periodJ\x04\x08\x04\x10\x05\"\xd8\x03\n\rAlterConsumer\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1a\n\rset_important\x18\x02 \x01(\x08H\x01\x88\x01\x01\x12\x31\n\rset_read_from\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x38\n\x14set_supported_codecs\x18\x05 \x01(\x0b\x32\x1a.Ydb.Topic.SupportedCodecs\x12G\n\x10\x61lter_attributes\x18\x06 \x03(\x0b\x32-.Ydb.Topic.AlterConsumer.AlterAttributesEntry\x12<\n\x17set_availability_period\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00\x12;\n\x19reset_availability_period\x18\x08 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00\x1a\x36\n\x14\x41lterAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x1c\n\x1a\x61vailability_period_actionB\x10\n\x0e_set_importantJ\x04\x08\x04\x10\x05\"\xdc\x01\n\x14PartitioningSettings\x12\'\n\x15min_active_partitions\x18\x01 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0\x12\'\n\x15max_active_partitions\x18\x03 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0\x12)\n\x15partition_count_limit\x18\x02 \x01(\x03\x42\n\x18\x01\xb2\xe6*\x04>= 0\x12G\n\x1a\x61uto_partitioning_settings\x18\x04 \x01(\x0b\x32#.Ydb.Topic.AutoPartitioningSettings\"\x9f\x01\n\x18\x41utoPartitioningSettings\x12\x35\n\x08strategy\x18\x01 \x01(\x0e\x32#.Ydb.Topic.AutoPartitioningStrategy\x12L\n\x15partition_write_speed\x18\x02 \x01(\x0b\x32-.Ydb.Topic.AutoPartitioningWriteSpeedStrategy\"\xb3\x01\n\"AutoPartitioningWriteSpeedStrategy\x12\x37\n\x14stabilization_window\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\x12(\n\x16up_utilization_percent\x18\x02 \x01(\x05\x42\x08\xb2\xe6*\x04>= 0\x12*\n\x18\x64own_utilization_percent\x18\x03 \x01(\x05\x42\x08\xb2\xe6*\x04>= 0\"\x8b\x03\n\x19\x41lterPartitioningSettings\x12\x30\n\x19set_min_active_partitions\x18\x01 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0H\x00\x88\x01\x01\x12\x30\n\x19set_max_active_partitions\x18\x03 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0H\x01\x88\x01\x01\x12\x32\n\x19set_partition_count_limit\x18\x02 \x01(\x03\x42\n\x18\x01\xb2\xe6*\x04>= 0H\x02\x88\x01\x01\x12W\n alter_auto_partitioning_settings\x18\x04 \x01(\x0b\x32(.Ydb.Topic.AlterAutoPartitioningSettingsH\x03\x88\x01\x01\x42\x1c\n\x1a_set_min_active_partitionsB\x1c\n\x1a_set_max_active_partitionsB\x1c\n\x1a_set_partition_count_limitB#\n!_alter_auto_partitioning_settings\"\xea\x01\n\x1d\x41lterAutoPartitioningSettings\x12>\n\x0cset_strategy\x18\x01 \x01(\x0e\x32#.Ydb.Topic.AutoPartitioningStrategyH\x00\x88\x01\x01\x12Z\n\x19set_partition_write_speed\x18\x02 \x01(\x0b\x32\x32.Ydb.Topic.AlterAutoPartitioningWriteSpeedStrategyH\x01\x88\x01\x01\x42\x0f\n\r_set_strategyB\x1c\n\x1a_set_partition_write_speed\"\xb0\x02\n\'AlterAutoPartitioningWriteSpeedStrategy\x12@\n\x18set_stabilization_window\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00\x88\x01\x01\x12\x31\n\x1aset_up_utilization_percent\x18\x02 \x01(\x05\x42\x08\xb2\xe6*\x04>= 0H\x01\x88\x01\x01\x12\x33\n\x1cset_down_utilization_percent\x18\x03 \x01(\x05\x42\x08\xb2\xe6*\x04>= 0H\x02\x88\x01\x01\x42\x1b\n\x19_set_stabilization_windowB\x1d\n\x1b_set_up_utilization_percentB\x1f\n\x1d_set_down_utilization_percent\"\xf6\x04\n\x12\x43reateTopicRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12>\n\x15partitioning_settings\x18\x03 \x01(\x0b\x32\x1f.Ydb.Topic.PartitioningSettings\x12\x33\n\x10retention_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12&\n\x14retention_storage_mb\x18\x05 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0\x12\x34\n\x10supported_codecs\x18\x07 \x01(\x0b\x32\x1a.Ydb.Topic.SupportedCodecs\x12\x38\n&partition_write_speed_bytes_per_second\x18\x08 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0\x12-\n\x1bpartition_write_burst_bytes\x18\t \x01(\x03\x42\x08\xb2\xe6*\x04>= 0\x12\x41\n\nattributes\x18\n \x03(\x0b\x32-.Ydb.Topic.CreateTopicRequest.AttributesEntry\x12/\n\tconsumers\x18\x0b \x03(\x0b\x32\x13.Ydb.Topic.ConsumerB\x07\x9a\xe6*\x03\x18\xb8\x17\x12.\n\rmetering_mode\x18\x0c \x01(\x0e\x32\x17.Ydb.Topic.MeteringMode\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01J\x04\x08\x06\x10\x07\"C\n\x13\x43reateTopicResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x13\n\x11\x43reateTopicResult\"8\n\x11PartitionLocation\x12\x0f\n\x07node_id\x18\x01 \x01(\x05\x12\x12\n\ngeneration\x18\x02 \x01(\x03\"\x90\x01\n\x14\x44\x65scribeTopicRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x15\n\rinclude_stats\x18\x03 \x01(\x08\x12\x18\n\x10include_location\x18\x04 \x01(\x08\"E\n\x15\x44\x65scribeTopicResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"_\n\x11PartitionKeyRange\x12\x17\n\nfrom_bound\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x12\x15\n\x08to_bound\x18\x02 \x01(\x0cH\x01\x88\x01\x01\x42\r\n\x0b_from_boundB\x0b\n\t_to_bound\"\xfa\t\n\x13\x44\x65scribeTopicResult\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x11.Ydb.Scheme.Entry\x12>\n\x15partitioning_settings\x18\x02 \x01(\x0b\x32\x1f.Ydb.Topic.PartitioningSettings\x12@\n\npartitions\x18\x03 \x03(\x0b\x32,.Ydb.Topic.DescribeTopicResult.PartitionInfo\x12\x33\n\x10retention_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x1c\n\x14retention_storage_mb\x18\x05 \x01(\x03\x12\x34\n\x10supported_codecs\x18\x07 \x01(\x0b\x32\x1a.Ydb.Topic.SupportedCodecs\x12.\n&partition_write_speed_bytes_per_second\x18\x08 \x01(\x03\x12\x33\n+partition_total_read_speed_bytes_per_second\x18\x0e \x01(\x03\x12\x36\n.partition_consumer_read_speed_bytes_per_second\x18\x0f \x01(\x03\x12#\n\x1bpartition_write_burst_bytes\x18\t \x01(\x03\x12\x42\n\nattributes\x18\n \x03(\x0b\x32..Ydb.Topic.DescribeTopicResult.AttributesEntry\x12&\n\tconsumers\x18\x0b \x03(\x0b\x32\x13.Ydb.Topic.Consumer\x12.\n\rmetering_mode\x18\x0c \x01(\x0e\x32\x17.Ydb.Topic.MeteringMode\x12>\n\x0btopic_stats\x18\r \x01(\x0b\x32).Ydb.Topic.DescribeTopicResult.TopicStats\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x8f\x02\n\rPartitionInfo\x12\x14\n\x0cpartition_id\x18\x01 \x01(\x03\x12\x0e\n\x06\x61\x63tive\x18\x02 \x01(\x08\x12\x1b\n\x13\x63hild_partition_ids\x18\x03 \x03(\x03\x12\x1c\n\x14parent_partition_ids\x18\x04 \x03(\x03\x12\x32\n\x0fpartition_stats\x18\x05 \x01(\x0b\x32\x19.Ydb.Topic.PartitionStats\x12\x38\n\x12partition_location\x18\x06 \x01(\x0b\x32\x1c.Ydb.Topic.PartitionLocation\x12/\n\tkey_range\x18\x07 \x01(\x0b\x32\x1c.Ydb.Topic.PartitionKeyRange\x1a\xcd\x01\n\nTopicStats\x12\x18\n\x10store_size_bytes\x18\x01 \x01(\x03\x12\x37\n\x13min_last_write_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12max_write_time_lag\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x35\n\rbytes_written\x18\x04 \x01(\x0b\x32\x1e.Ydb.Topic.MultipleWindowsStatJ\x04\x08\x06\x10\x07\"\xaa\x01\n\x18\x44\x65scribePartitionRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x14\n\x0cpartition_id\x18\x03 \x01(\x03\x12\x15\n\rinclude_stats\x18\x04 \x01(\x08\x12\x18\n\x10include_location\x18\x05 \x01(\x08\"I\n\x19\x44\x65scribePartitionResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"Z\n\x17\x44\x65scribePartitionResult\x12?\n\tpartition\x18\x01 \x01(\x0b\x32,.Ydb.Topic.DescribeTopicResult.PartitionInfo\"\xa5\x01\n\x17\x44\x65scribeConsumerRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x10\n\x08\x63onsumer\x18\x03 \x01(\t\x12\x15\n\rinclude_stats\x18\x04 \x01(\x08\x12\x18\n\x10include_location\x18\x05 \x01(\x08\"H\n\x18\x44\x65scribeConsumerResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\xd3\x07\n\x16\x44\x65scribeConsumerResult\x12\x1f\n\x04self\x18\x01 \x01(\x0b\x32\x11.Ydb.Scheme.Entry\x12%\n\x08\x63onsumer\x18\x02 \x01(\x0b\x32\x13.Ydb.Topic.Consumer\x12\x43\n\npartitions\x18\x03 \x03(\x0b\x32/.Ydb.Topic.DescribeConsumerResult.PartitionInfo\x1a\xba\x02\n\rPartitionInfo\x12\x14\n\x0cpartition_id\x18\x01 \x01(\x03\x12\x0e\n\x06\x61\x63tive\x18\x02 \x01(\x08\x12\x1b\n\x13\x63hild_partition_ids\x18\x03 \x03(\x03\x12\x1c\n\x14parent_partition_ids\x18\x04 \x03(\x03\x12\x32\n\x0fpartition_stats\x18\x05 \x01(\x0b\x32\x19.Ydb.Topic.PartitionStats\x12Z\n\x18partition_consumer_stats\x18\x06 \x01(\x0b\x32\x38.Ydb.Topic.DescribeConsumerResult.PartitionConsumerStats\x12\x38\n\x12partition_location\x18\x07 \x01(\x0b\x32\x1c.Ydb.Topic.PartitionLocation\x1a\xee\x03\n\x16PartitionConsumerStats\x12\x18\n\x10last_read_offset\x18\x01 \x01(\x03\x12\x18\n\x10\x63ommitted_offset\x18\x02 \x01(\x03\x12\x17\n\x0fread_session_id\x18\x03 \x01(\t\x12\x46\n\"partition_read_session_create_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0elast_read_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x11max_read_time_lag\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x35\n\x12max_write_time_lag\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x39\n\x16max_committed_time_lag\x18\r \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x32\n\nbytes_read\x18\x08 \x01(\x0b\x32\x1e.Ydb.Topic.MultipleWindowsStat\x12\x13\n\x0breader_name\x18\x0b \x01(\t\x12\x1a\n\x12\x63onnection_node_id\x18\x0c \x01(\x05\"\xa0\x02\n\x0ePartitionStats\x12\x32\n\x11partition_offsets\x18\x01 \x01(\x0b\x32\x17.Ydb.Topic.OffsetsRange\x12\x18\n\x10store_size_bytes\x18\x02 \x01(\x03\x12\x33\n\x0flast_write_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12max_write_time_lag\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x35\n\rbytes_written\x18\x05 \x01(\x0b\x32\x1e.Ydb.Topic.MultipleWindowsStat\x12\x1d\n\x11partition_node_id\x18\x08 \x01(\x05\x42\x02\x18\x01\"\x87\x07\n\x11\x41lterTopicRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\x12I\n\x1b\x61lter_partitioning_settings\x18\x03 \x01(\x0b\x32$.Ydb.Topic.AlterPartitioningSettings\x12\x37\n\x14set_retention_period\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12/\n\x18set_retention_storage_mb\x18\x05 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0H\x00\x88\x01\x01\x12\x38\n\x14set_supported_codecs\x18\x07 \x01(\x0b\x32\x1a.Ydb.Topic.SupportedCodecs\x12\x41\n*set_partition_write_speed_bytes_per_second\x18\x08 \x01(\x03\x42\x08\xb2\xe6*\x04>= 0H\x01\x88\x01\x01\x12\x36\n\x1fset_partition_write_burst_bytes\x18\t \x01(\x03\x42\x08\xb2\xe6*\x04>= 0H\x02\x88\x01\x01\x12K\n\x10\x61lter_attributes\x18\n \x03(\x0b\x32\x31.Ydb.Topic.AlterTopicRequest.AlterAttributesEntry\x12\x33\n\radd_consumers\x18\x0b \x03(\x0b\x32\x13.Ydb.Topic.ConsumerB\x07\x9a\xe6*\x03\x18\xb8\x17\x12\x1f\n\x0e\x64rop_consumers\x18\x0c \x03(\tB\x07\x9a\xe6*\x03\x18\xb8\x17\x12:\n\x0f\x61lter_consumers\x18\r \x03(\x0b\x32\x18.Ydb.Topic.AlterConsumerB\x07\x9a\xe6*\x03\x18\xb8\x17\x12\x32\n\x11set_metering_mode\x18\x0e \x01(\x0e\x32\x17.Ydb.Topic.MeteringMode\x1a\x36\n\x14\x41lterAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x1b\n\x19_set_retention_storage_mbB-\n+_set_partition_write_speed_bytes_per_secondB\"\n _set_partition_write_burst_bytesJ\x04\x08\x06\x10\x07\"B\n\x12\x41lterTopicResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x12\n\x10\x41lterTopicResult\"[\n\x10\x44ropTopicRequest\x12\x39\n\x10operation_params\x18\x01 \x01(\x0b\x32\x1f.Ydb.Operations.OperationParams\x12\x0c\n\x04path\x18\x02 \x01(\t\"A\n\x11\x44ropTopicResponse\x12,\n\toperation\x18\x01 \x01(\x0b\x32\x19.Ydb.Operations.Operation\"\x11\n\x0f\x44ropTopicResult*\x83\x01\n\x05\x43odec\x12\x15\n\x11\x43ODEC_UNSPECIFIED\x10\x00\x12\r\n\tCODEC_RAW\x10\x01\x12\x0e\n\nCODEC_GZIP\x10\x02\x12\x0e\n\nCODEC_LZOP\x10\x03\x12\x0e\n\nCODEC_ZSTD\x10\x04\x12\x11\n\x0c\x43ODEC_CUSTOM\x10\x90N\"\x05\x08\x05\x10\x8fN\"\n\x08\xa0\x9c\x01\x10\xff\xff\xff\xff\x07*\xf1\x01\n\x18\x41utoPartitioningStrategy\x12*\n&AUTO_PARTITIONING_STRATEGY_UNSPECIFIED\x10\x00\x12\'\n#AUTO_PARTITIONING_STRATEGY_DISABLED\x10\x01\x12\'\n#AUTO_PARTITIONING_STRATEGY_SCALE_UP\x10\x02\x12\x30\n,AUTO_PARTITIONING_STRATEGY_SCALE_UP_AND_DOWN\x10\x03\x12%\n!AUTO_PARTITIONING_STRATEGY_PAUSED\x10\x04*s\n\x0cMeteringMode\x12\x1d\n\x19METERING_MODE_UNSPECIFIED\x10\x00\x12#\n\x1fMETERING_MODE_RESERVED_CAPACITY\x10\x01\x12\x1f\n\x1bMETERING_MODE_REQUEST_UNITS\x10\x02\x42S\n\x14tech.ydb.proto.topicZ8github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Topic\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protos.ydb_topic_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024tech.ydb.proto.topicZ8github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Topic\370\001\001'
  _SUPPORTEDCODECS.fields_by_name['codecs']._options = None
  _SUPPORTEDCODECS.fields_by_name['codecs']._serialized_options = b'\262\346*\n[1; 19999]\232\346*\002\030d'
  _UPDATETOKENREQUEST.fields_by_name['token']._options = None
  _UPDATETOKENREQUEST.fields_by_name['token']._serialized_options = b'\270\346*\001'
  _STREAMWRITEMESSAGE_INITREQUEST_WRITESESSIONMETAENTRY._options = None
  _STREAMWRITEMESSAGE_INITREQUEST_WRITESESSIONMETAENTRY._serialized_options = b'8\001'
  _STREAMWRITEMESSAGE_INITREQUEST.fields_by_name['producer_id']._options = None
  _STREAMWRITEMESSAGE_INITREQUEST.fields_by_name['producer_id']._serialized_options = b'\242\346*\003\030\200\020'
  _STREAMWRITEMESSAGE_INITREQUEST.fields_by_name['message_group_id']._options = None
  _STREAMWRITEMESSAGE_INITREQUEST.fields_by_name['message_group_id']._serialized_options = b'\242\346*\003\030\200\020'
  _STREAMWRITEMESSAGE_WRITEREQUEST_MESSAGEDATA.fields_by_name['message_group_id']._options = None
  _STREAMWRITEMESSAGE_WRITEREQUEST_MESSAGEDATA.fields_by_name['message_group_id']._serialized_options = b'\242\346*\003\030\200\020'
  _STREAMWRITEMESSAGE_WRITEREQUEST_MESSAGEDATA.fields_by_name['metadata_items']._options = None
  _STREAMWRITEMESSAGE_WRITEREQUEST_MESSAGEDATA.fields_by_name['metadata_items']._serialized_options = b'\232\346*\003\030\350\007'
  _STREAMREADMESSAGE_READRESPONSE_MESSAGEDATA.fields_by_name['message_group_id']._options = None
  _STREAMREADMESSAGE_READRESPONSE_MESSAGEDATA.fields_by_name['message_group_id']._serialized_options = b'\242\346*\003\030\200\020'
  _STREAMREADMESSAGE_READRESPONSE_BATCH_WRITESESSIONMETAENTRY._options = None
  _STREAMREADMESSAGE_READRESPONSE_BATCH_WRITESESSIONMETAENTRY._serialized_options = b'8\001'
  _STREAMREADMESSAGE_READRESPONSE_BATCH.fields_by_name['producer_id']._options = None
  _STREAMREADMESSAGE_READRESPONSE_BATCH.fields_by_name['producer_id']._serialized_options = b'\242\346*\003\030\200\020'
  _CONSUMER_ATTRIBUTESENTRY._options = None
  _CONSUMER_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _ALTERCONSUMER_ALTERATTRIBUTESENTRY._options = None
  _ALTERCONSUMER_ALTERATTRIBUTESENTRY._serialized_options = b'8\001'
  _PARTITIONINGSETTINGS.fields_by_name['min_active_partitions']._options = None
  _PARTITIONINGSETTINGS.fields_by_name['min_active_partitions']._serialized_options = b'\262\346*\004>= 0'
  _PARTITIONINGSETTINGS.fields_by_name['max_active_partitions']._options = None
  _PARTITIONINGSETTINGS.fields_by_name['max_active_partitions']._serialized_options = b'\262\346*\004>= 0'
  _PARTITIONINGSETTINGS.fields_by_name['partition_count_limit']._options = None
  _PARTITIONINGSETTINGS.fields_by_name['partition_count_limit']._serialized_options = b'\030\001\262\346*\004>= 0'
  _AUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['up_utilization_percent']._options = None
  _AUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['up_utilization_percent']._serialized_options = b'\262\346*\004>= 0'
  _AUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['down_utilization_percent']._options = None
  _AUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['down_utilization_percent']._serialized_options = b'\262\346*\004>= 0'
  _ALTERPARTITIONINGSETTINGS.fields_by_name['set_min_active_partitions']._options = None
  _ALTERPARTITIONINGSETTINGS.fields_by_name['set_min_active_partitions']._serialized_options = b'\262\346*\004>= 0'
  _ALTERPARTITIONINGSETTINGS.fields_by_name['set_max_active_partitions']._options = None
  _ALTERPARTITIONINGSETTINGS.fields_by_name['set_max_active_partitions']._serialized_options = b'\262\346*\004>= 0'
  _ALTERPARTITIONINGSETTINGS.fields_by_name['set_partition_count_limit']._options = None
  _ALTERPARTITIONINGSETTINGS.fields_by_name['set_partition_count_limit']._serialized_options = b'\030\001\262\346*\004>= 0'
  _ALTERAUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['set_up_utilization_percent']._options = None
  _ALTERAUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['set_up_utilization_percent']._serialized_options = b'\262\346*\004>= 0'
  _ALTERAUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['set_down_utilization_percent']._options = None
  _ALTERAUTOPARTITIONINGWRITESPEEDSTRATEGY.fields_by_name['set_down_utilization_percent']._serialized_options = b'\262\346*\004>= 0'
  _CREATETOPICREQUEST_ATTRIBUTESENTRY._options = None
  _CREATETOPICREQUEST_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _CREATETOPICREQUEST.fields_by_name['retention_storage_mb']._options = None
  _CREATETOPICREQUEST.fields_by_name['retention_storage_mb']._serialized_options = b'\262\346*\004>= 0'
  _CREATETOPICREQUEST.fields_by_name['partition_write_speed_bytes_per_second']._options = None
  _CREATETOPICREQUEST.fields_by_name['partition_write_speed_bytes_per_second']._serialized_options = b'\262\346*\004>= 0'
  _CREATETOPICREQUEST.fields_by_name['partition_write_burst_bytes']._options = None
  _CREATETOPICREQUEST.fields_by_name['partition_write_burst_bytes']._serialized_options = b'\262\346*\004>= 0'
  _CREATETOPICREQUEST.fields_by_name['consumers']._options = None
  _CREATETOPICREQUEST.fields_by_name['consumers']._serialized_options = b'\232\346*\003\030\270\027'
  _DESCRIBETOPICRESULT_ATTRIBUTESENTRY._options = None
  _DESCRIBETOPICRESULT_ATTRIBUTESENTRY._serialized_options = b'8\001'
  _PARTITIONSTATS.fields_by_name['partition_node_id']._options = None
  _PARTITIONSTATS.fields_by_name['partition_node_id']._serialized_options = b'\030\001'
  _ALTERTOPICREQUEST_ALTERATTRIBUTESENTRY._options = None
  _ALTERTOPICREQUEST_ALTERATTRIBUTESENTRY._serialized_options = b'8\001'
  _ALTERTOPICREQUEST.fields_by_name['set_retention_storage_mb']._options = None
  _ALTERTOPICREQUEST.fields_by_name['set_retention_storage_mb']._serialized_options = b'\262\346*\004>= 0'
  _ALTERTOPICREQUEST.fields_by_name['set_partition_write_speed_bytes_per_second']._options = None
  _ALTERTOPICREQUEST.fields_by_name['set_partition_write_speed_bytes_per_second']._serialized_options = b'\262\346*\004>= 0'
  _ALTERTOPICREQUEST.fields_by_name['set_partition_write_burst_bytes']._options = None
  _ALTERTOPICREQUEST.fields_by_name['set_partition_write_burst_bytes']._serialized_options = b'\262\346*\004>= 0'
  _ALTERTOPICREQUEST.fields_by_name['add_consumers']._options = None
  _ALTERTOPICREQUEST.fields_by_name['add_consumers']._serialized_options = b'\232\346*\003\030\270\027'
  _ALTERTOPICREQUEST.fields_by_name['drop_consumers']._options = None
  _ALTERTOPICREQUEST.fields_by_name['drop_consumers']._serialized_options = b'\232\346*\003\030\270\027'
  _ALTERTOPICREQUEST.fields_by_name['alter_consumers']._options = None
  _ALTERTOPICREQUEST.fields_by_name['alter_consumers']._serialized_options = b'\232\346*\003\030\270\027'
  _CODEC._serialized_start=18453
  _CODEC._serialized_end=18584
  _AUTOPARTITIONINGSTRATEGY._serialized_start=18587
  _AUTOPARTITIONINGSTRATEGY._serialized_end=18828
  _METERINGMODE._serialized_start=18830
  _METERINGMODE._serialized_end=18945
  _SUPPORTEDCODECS._serialized_start=320
  _SUPPORTEDCODECS._serialized_end=375
  _OFFSETSRANGE._serialized_start=377
  _OFFSETSRANGE._serialized_end=419
  _UPDATETOKENREQUEST._serialized_start=421
  _UPDATETOKENREQUEST._serialized_end=462
  _UPDATETOKENRESPONSE._serialized_start=464
  _UPDATETOKENRESPONSE._serialized_end=485
  _PARTITIONWITHGENERATION._serialized_start=487
  _PARTITIONWITHGENERATION._serialized_end=554
  _METADATAITEM._serialized_start=556
  _METADATAITEM._serialized_end=598
  _STREAMWRITEMESSAGE._serialized_start=601
  _STREAMWRITEMESSAGE._serialized_end=3191
  _STREAMWRITEMESSAGE_FROMCLIENT._serialized_start=624
  _STREAMWRITEMESSAGE_FROMCLIENT._serialized_end=853
  _STREAMWRITEMESSAGE_FROMSERVER._serialized_start=856
  _STREAMWRITEMESSAGE_FROMSERVER._serialized_end=1175
  _STREAMWRITEMESSAGE_INITREQUEST._serialized_start=1178
  _STREAMWRITEMESSAGE_INITREQUEST._serialized_end=1560
  _STREAMWRITEMESSAGE_INITREQUEST_WRITESESSIONMETAENTRY._serialized_start=1489
  _STREAMWRITEMESSAGE_INITREQUEST_WRITESESSIONMETAENTRY._serialized_end=1544
  _STREAMWRITEMESSAGE_INITRESPONSE._serialized_start=1563
  _STREAMWRITEMESSAGE_INITRESPONSE._serialized_end=1694
  _STREAMWRITEMESSAGE_WRITEREQUEST._serialized_start=1697
  _STREAMWRITEMESSAGE_WRITEREQUEST._serialized_end=2185
  _STREAMWRITEMESSAGE_WRITEREQUEST_MESSAGEDATA._serialized_start=1852
  _STREAMWRITEMESSAGE_WRITEREQUEST_MESSAGEDATA._serialized_end=2178
  _STREAMWRITEMESSAGE_WRITERESPONSE._serialized_start=2188
  _STREAMWRITEMESSAGE_WRITERESPONSE._serialized_end=3191
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK._serialized_start=2383
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK._serialized_end=2887
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_WRITTEN._serialized_start=2664
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_WRITTEN._serialized_end=2689
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_SKIPPED._serialized_start=2692
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_SKIPPED._serialized_end=2848
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_SKIPPED_REASON._serialized_start=2788
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_SKIPPED_REASON._serialized_end=2848
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_WRITTENINTX._serialized_start=2850
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITEACK_WRITTENINTX._serialized_end=2863
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITESTATISTICS._serialized_start=2890
  _STREAMWRITEMESSAGE_WRITERESPONSE_WRITESTATISTICS._serialized_end=3191
  _STREAMREADMESSAGE._serialized_start=3194
  _STREAMREADMESSAGE._serialized_end=7702
  _STREAMREADMESSAGE_PARTITIONSESSION._serialized_start=3215
  _STREAMREADMESSAGE_PARTITIONSESSION._serialized_end=3299
  _STREAMREADMESSAGE_FROMCLIENT._serialized_start=3302
  _STREAMREADMESSAGE_FROMCLIENT._serialized_end=3991
  _STREAMREADMESSAGE_FROMSERVER._serialized_start=3994
  _STREAMREADMESSAGE_FROMSERVER._serialized_end=4874
  _STREAMREADMESSAGE_INITREQUEST._serialized_start=4877
  _STREAMREADMESSAGE_INITREQUEST._serialized_end=5225
  _STREAMREADMESSAGE_INITREQUEST_TOPICREADSETTINGS._serialized_start=5078
  _STREAMREADMESSAGE_INITREQUEST_TOPICREADSETTINGS._serialized_end=5225
  _STREAMREADMESSAGE_INITRESPONSE._serialized_start=5227
  _STREAMREADMESSAGE_INITRESPONSE._serialized_end=5261
  _STREAMREADMESSAGE_READREQUEST._serialized_start=5263
  _STREAMREADMESSAGE_READREQUEST._serialized_end=5296
  _STREAMREADMESSAGE_READRESPONSE._serialized_start=5299
  _STREAMREADMESSAGE_READRESPONSE._serialized_end=6084
  _STREAMREADMESSAGE_READRESPONSE_MESSAGEDATA._serialized_start=5417
  _STREAMREADMESSAGE_READRESPONSE_MESSAGEDATA._serialized_end=5635
  _STREAMREADMESSAGE_READRESPONSE_BATCH._serialized_start=5638
  _STREAMREADMESSAGE_READRESPONSE_BATCH._serialized_end=5971
  _STREAMREADMESSAGE_READRESPONSE_BATCH_WRITESESSIONMETAENTRY._serialized_start=1489
  _STREAMREADMESSAGE_READRESPONSE_BATCH_WRITESESSIONMETAENTRY._serialized_end=1544
  _STREAMREADMESSAGE_READRESPONSE_PARTITIONDATA._serialized_start=5973
  _STREAMREADMESSAGE_READRESPONSE_PARTITIONDATA._serialized_end=6084
  _STREAMREADMESSAGE_COMMITOFFSETREQUEST._serialized_start=6087
  _STREAMREADMESSAGE_COMMITOFFSETREQUEST._serialized_end=6301
  _STREAMREADMESSAGE_COMMITOFFSETREQUEST_PARTITIONCOMMITOFFSET._serialized_start=6206
  _STREAMREADMESSAGE_COMMITOFFSETREQUEST_PARTITIONCOMMITOFFSET._serialized_end=6301
  _STREAMREADMESSAGE_COMMITOFFSETRESPONSE._serialized_start=6304
  _STREAMREADMESSAGE_COMMITOFFSETRESPONSE._serialized_end=6524
  _STREAMREADMESSAGE_COMMITOFFSETRESPONSE_PARTITIONCOMMITTEDOFFSET._serialized_start=6442
  _STREAMREADMESSAGE_COMMITOFFSETRESPONSE_PARTITIONCOMMITTEDOFFSET._serialized_end=6524
  _STREAMREADMESSAGE_PARTITIONSESSIONSTATUSREQUEST._serialized_start=6526
  _STREAMREADMESSAGE_PARTITIONSESSIONSTATUSREQUEST._serialized_end=6587
  _STREAMREADMESSAGE_PARTITIONSESSIONSTATUSRESPONSE._serialized_start=6590
  _STREAMREADMESSAGE_PARTITIONSESSIONSTATUSRESPONSE._serialized_end=6793
  _STREAMREADMESSAGE_STARTPARTITIONSESSIONREQUEST._serialized_start=6796
  _STREAMREADMESSAGE_STARTPARTITIONSESSIONREQUEST._serialized_end=7036
  _STREAMREADMESSAGE_STARTPARTITIONSESSIONRESPONSE._serialized_start=7039
  _STREAMREADMESSAGE_STARTPARTITIONSESSIONRESPONSE._serialized_end=7188
  _STREAMREADMESSAGE_STOPPARTITIONSESSIONREQUEST._serialized_start=7191
  _STREAMREADMESSAGE_STOPPARTITIONSESSIONREQUEST._serialized_end=7323
  _STREAMREADMESSAGE_STOPPARTITIONSESSIONRESPONSE._serialized_start=7325
  _STREAMREADMESSAGE_STOPPARTITIONSESSIONRESPONSE._serialized_end=7403
  _STREAMREADMESSAGE_UPDATEPARTITIONSESSION._serialized_start=7405
  _STREAMREADMESSAGE_UPDATEPARTITIONSESSION._serialized_end=7517
  _STREAMREADMESSAGE_DIRECTREADACK._serialized_start=7519
  _STREAMREADMESSAGE_DIRECTREADACK._serialized_end=7588
  _STREAMREADMESSAGE_ENDPARTITIONSESSION._serialized_start=7590
  _STREAMREADMESSAGE_ENDPARTITIONSESSION._serialized_end=7702
  _STREAMDIRECTREADMESSAGE._serialized_start=7705
  _STREAMDIRECTREADMESSAGE._serialized_end=9380
  _STREAMDIRECTREADMESSAGE_FROMCLIENT._serialized_start=7733
  _STREAMDIRECTREADMESSAGE_FROMCLIENT._serialized_end=8029
  _STREAMDIRECTREADMESSAGE_FROMSERVER._serialized_start=8032
  _STREAMDIRECTREADMESSAGE_FROMSERVER._serialized_end=8618
  _STREAMDIRECTREADMESSAGE_INITREQUEST._serialized_start=8621
  _STREAMDIRECTREADMESSAGE_INITREQUEST._serialized_end=8803
  _STREAMDIRECTREADMESSAGE_INITREQUEST_TOPICREADSETTINGS._serialized_start=5078
  _STREAMDIRECTREADMESSAGE_INITREQUEST_TOPICREADSETTINGS._serialized_end=5111
  _STREAMDIRECTREADMESSAGE_INITRESPONSE._serialized_start=1563
  _STREAMDIRECTREADMESSAGE_INITRESPONSE._serialized_end=1577
  _STREAMDIRECTREADMESSAGE_STARTDIRECTREADPARTITIONSESSIONREQUEST._serialized_start=8821
  _STREAMDIRECTREADMESSAGE_STARTDIRECTREADPARTITIONSESSIONREQUEST._serialized_end=8940
  _STREAMDIRECTREADMESSAGE_STARTDIRECTREADPARTITIONSESSIONRESPONSE._serialized_start=8942
  _STREAMDIRECTREADMESSAGE_STARTDIRECTREADPARTITIONSESSIONRESPONSE._serialized_end=9033
  _STREAMDIRECTREADMESSAGE_STOPDIRECTREADPARTITIONSESSION._serialized_start=9036
  _STREAMDIRECTREADMESSAGE_STOPDIRECTREADPARTITIONSESSION._serialized_end=9202
  _STREAMDIRECTREADMESSAGE_DIRECTREADRESPONSE._serialized_start=9205
  _STREAMDIRECTREADMESSAGE_DIRECTREADRESPONSE._serialized_end=9380
  _TRANSACTIONIDENTITY._serialized_start=9382
  _TRANSACTIONIDENTITY._serialized_end=9432
  _UPDATEOFFSETSINTRANSACTIONREQUEST._serialized_start=9435
  _UPDATEOFFSETSINTRANSACTIONREQUEST._serialized_end=9887
  _UPDATEOFFSETSINTRANSACTIONREQUEST_TOPICOFFSETS._serialized_start=9669
  _UPDATEOFFSETSINTRANSACTIONREQUEST_TOPICOFFSETS._serialized_end=9887
  _UPDATEOFFSETSINTRANSACTIONREQUEST_TOPICOFFSETS_PARTITIONOFFSETS._serialized_start=9795
  _UPDATEOFFSETSINTRANSACTIONREQUEST_TOPICOFFSETS_PARTITIONOFFSETS._serialized_end=9887
  _UPDATEOFFSETSINTRANSACTIONRESPONSE._serialized_start=9889
  _UPDATEOFFSETSINTRANSACTIONRESPONSE._serialized_end=9971
  _UPDATEOFFSETSINTRANSACTIONRESULT._serialized_start=9973
  _UPDATEOFFSETSINTRANSACTIONRESULT._serialized_end=10007
  _COMMITOFFSETREQUEST._serialized_start=10010
  _COMMITOFFSETREQUEST._serialized_end=10185
  _COMMITOFFSETRESPONSE._serialized_start=10187
  _COMMITOFFSETRESPONSE._serialized_end=10255
  _COMMITOFFSETRESULT._serialized_start=10257
  _COMMITOFFSETRESULT._serialized_end=10277
  _MULTIPLEWINDOWSSTAT._serialized_start=10279
  _MULTIPLEWINDOWSSTAT._serialized_end=10355
  _CONSUMER._serialized_start=10358
  _CONSUMER._serialized_end=11065
  _CONSUMER_ATTRIBUTESENTRY._serialized_start=10681
  _CONSUMER_ATTRIBUTESENTRY._serialized_end=10730
  _CONSUMER_CONSUMERSTATS._serialized_start=10733
  _CONSUMER_CONSUMERSTATS._serialized_end=11035
  _ALTERCONSUMER._serialized_start=11068
  _ALTERCONSUMER._serialized_end=11540
  _ALTERCONSUMER_ALTERATTRIBUTESENTRY._serialized_start=11432
  _ALTERCONSUMER_ALTERATTRIBUTESENTRY._serialized_end=11486
  _PARTITIONINGSETTINGS._serialized_start=11543
  _PARTITIONINGSETTINGS._serialized_end=11763
  _AUTOPARTITIONINGSETTINGS._serialized_start=11766
  _AUTOPARTITIONINGSETTINGS._serialized_end=11925
  _AUTOPARTITIONINGWRITESPEEDSTRATEGY._serialized_start=11928
  _AUTOPARTITIONINGWRITESPEEDSTRATEGY._serialized_end=12107
  _ALTERPARTITIONINGSETTINGS._serialized_start=12110
  _ALTERPARTITIONINGSETTINGS._serialized_end=12505
  _ALTERAUTOPARTITIONINGSETTINGS._serialized_start=12508
  _ALTERAUTOPARTITIONINGSETTINGS._serialized_end=12742
  _ALTERAUTOPARTITIONINGWRITESPEEDSTRATEGY._serialized_start=12745
  _ALTERAUTOPARTITIONINGWRITESPEEDSTRATEGY._serialized_end=13049
  _CREATETOPICREQUEST._serialized_start=13052
  _CREATETOPICREQUEST._serialized_end=13682
  _CREATETOPICREQUEST_ATTRIBUTESENTRY._serialized_start=10681
  _CREATETOPICREQUEST_ATTRIBUTESENTRY._serialized_end=10730
  _CREATETOPICRESPONSE._serialized_start=13684
  _CREATETOPICRESPONSE._serialized_end=13751
  _CREATETOPICRESULT._serialized_start=13753
  _CREATETOPICRESULT._serialized_end=13772
  _PARTITIONLOCATION._serialized_start=13774
  _PARTITIONLOCATION._serialized_end=13830
  _DESCRIBETOPICREQUEST._serialized_start=13833
  _DESCRIBETOPICREQUEST._serialized_end=13977
  _DESCRIBETOPICRESPONSE._serialized_start=13979
  _DESCRIBETOPICRESPONSE._serialized_end=14048
  _PARTITIONKEYRANGE._serialized_start=14050
  _PARTITIONKEYRANGE._serialized_end=14145
  _DESCRIBETOPICRESULT._serialized_start=14148
  _DESCRIBETOPICRESULT._serialized_end=15422
  _DESCRIBETOPICRESULT_ATTRIBUTESENTRY._serialized_start=10681
  _DESCRIBETOPICRESULT_ATTRIBUTESENTRY._serialized_end=10730
  _DESCRIBETOPICRESULT_PARTITIONINFO._serialized_start=14937
  _DESCRIBETOPICRESULT_PARTITIONINFO._serialized_end=15208
  _DESCRIBETOPICRESULT_TOPICSTATS._serialized_start=15211
  _DESCRIBETOPICRESULT_TOPICSTATS._serialized_end=15416
  _DESCRIBEPARTITIONREQUEST._serialized_start=15425
  _DESCRIBEPARTITIONREQUEST._serialized_end=15595
  _DESCRIBEPARTITIONRESPONSE._serialized_start=15597
  _DESCRIBEPARTITIONRESPONSE._serialized_end=15670
  _DESCRIBEPARTITIONRESULT._serialized_start=15672
  _DESCRIBEPARTITIONRESULT._serialized_end=15762
  _DESCRIBECONSUMERREQUEST._serialized_start=15765
  _DESCRIBECONSUMERREQUEST._serialized_end=15930
  _DESCRIBECONSUMERRESPONSE._serialized_start=15932
  _DESCRIBECONSUMERRESPONSE._serialized_end=16004
  _DESCRIBECONSUMERRESULT._serialized_start=16007
  _DESCRIBECONSUMERRESULT._serialized_end=16986
  _DESCRIBECONSUMERRESULT_PARTITIONINFO._serialized_start=16175
  _DESCRIBECONSUMERRESULT_PARTITIONINFO._serialized_end=16489
  _DESCRIBECONSUMERRESULT_PARTITIONCONSUMERSTATS._serialized_start=16492
  _DESCRIBECONSUMERRESULT_PARTITIONCONSUMERSTATS._serialized_end=16986
  _PARTITIONSTATS._serialized_start=16989
  _PARTITIONSTATS._serialized_end=17277
  _ALTERTOPICREQUEST._serialized_start=17280
  _ALTERTOPICREQUEST._serialized_end=18183
  _ALTERTOPICREQUEST_ALTERATTRIBUTESENTRY._serialized_start=11432
  _ALTERTOPICREQUEST_ALTERATTRIBUTESENTRY._serialized_end=11486
  _ALTERTOPICRESPONSE._serialized_start=18185
  _ALTERTOPICRESPONSE._serialized_end=18251
  _ALTERTOPICRESULT._serialized_start=18253
  _ALTERTOPICRESULT._serialized_end=18271
  _DROPTOPICREQUEST._serialized_start=18273
  _DROPTOPICREQUEST._serialized_end=18364
  _DROPTOPICRESPONSE._serialized_start=18366
  _DROPTOPICRESPONSE._serialized_end=18431
  _DROPTOPICRESULT._serialized_start=18433
  _DROPTOPICRESULT._serialized_end=18450
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
import abc
import typing
import time
import os
from datetime import datetime

try:
    import jwt
except ImportError:
    jwt = None


class Token(abc.ABC):
    def __init__(self, token: str, token_type: str):
        self.token = token
        self.token_type = token_type


class TokenSource(abc.ABC):
    @abc.abstractmethod
    def token(self) -> Token:
        """
        :return: A Token object ready for exchange
        """
        pass


class FixedTokenSource(TokenSource):
    def __init__(self, token: str, token_type: str):
        self._token = Token(token, token_type)

    def token(self) -> Token:
        return self._token


class JwtTokenSource(TokenSource):
    def __init__(
        self,
        signing_method: str,
        private_key: typing.Optional[str] = None,
        private_key_file: typing.Optional[str] = None,
        key_id: typing.Optional[str] = None,
        issuer: typing.Optional[str] = None,
        subject: typing.Optional[str] = None,
        audience: typing.Union[typing.List[str], str, None] = None,
        id: typing.Optional[str] = None,
        token_ttl_seconds: int = 3600,
    ):
        assert jwt is not None, "Install pyjwt library to use jwt tokens"
        self._signing_method = signing_method
        self._key_id = key_id
        if private_key and private_key_file:
            raise Exception("JWT: both private_key and private_key_file are set")
        self._private_key = ""
        if private_key:
            self._private_key = private_key
        if private_key_file:
            private_key_file = os.path.expanduser(private_key_file)
            with open(private_key_file, "r") as key_file:
                self._private_key = key_file.read()
        self._issuer = issuer
        self._subject = subject
        self._audience = audience
        self._id = id
        self._token_ttl_seconds = token_ttl_seconds
        if not self._signing_method:
            raise Exception("JWT: no signing method specified")
        if not self._private_key:
            raise Exception("JWT: no private key specified")
        if self._token_ttl_seconds <= 0:
            raise Exception("JWT: invalid jwt token TTL")

    def token(self) -> Token:
        now = time.time()
        now_utc = datetime.utcfromtimestamp(now)
        exp_utc = datetime.utcfromtimestamp(now + self._token_ttl_seconds)
        payload = {
            "iat": now_utc,
            "exp": exp_utc,
        }
        if self._audience:
            payload["aud"] = self._audience
        if self._issuer:
            payload["iss"] = self._issuer
        if self._subject:
            payload["sub"] = self._subject
        if self._id:
            payload["jti"] = self._id

        headers = {
            "alg": self._signing_method,
            "typ": "JWT",
        }
        if self._key_id:
            headers["kid"] = self._key_id

        token = jwt.encode(
            key=self._private_key,
            algorithm=self._signing_method,
            headers=headers,
            payload=payload,
        )
        return Token(token, "urn:ietf:params:oauth:token-type:jwt")



import jwt
import concurrent.futures
import grpc
import time

import ydb.iam

from yandex.cloud.iam.v1 import iam_token_service_pb2_grpc
from yandex.cloud.iam.v1 import iam_token_service_pb2

SERVICE_ACCOUNT_ID = "sa_id"
ACCESS_KEY_ID = "key_id"
PRIVATE_KEY = "-----BEGIN PRIVATE KEY-----\nMIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQC75/JS3rMcLJxv\nFgpOzF5+2gH+Yig3RE2MTl9uwC0BZKAv6foYr7xywQyWIK+W1cBhz8R4LfFmZo2j\nM0aCvdRmNBdW0EDSTnHLxCsFhoQWLVq+bI5f5jzkcoiioUtaEpADPqwgVULVtN/n\nnPJiZ6/dU30C3jmR6+LUgEntUtWt3eq3xQIn5lG3zC1klBY/HxtfH5Hu8xBvwRQT\nJnh3UpPLj8XwSmriDgdrhR7o6umWyVuGrMKlLHmeivlfzjYtfzO1MOIMG8t2/zxG\nR+xb4Vwks73sH1KruH/0/JMXU97npwpe+Um+uXhpldPygGErEia7abyZB2gMpXqr\nWYKMo02NAgMBAAECggEAO0BpC5OYw/4XN/optu4/r91bupTGHKNHlsIR2rDzoBhU\nYLd1evpTQJY6O07EP5pYZx9mUwUdtU4KRJeDGO/1/WJYp7HUdtxwirHpZP0lQn77\nuccuX/QQaHLrPekBgz4ONk+5ZBqukAfQgM7fKYOLk41jgpeDbM2Ggb6QUSsJISEp\nzrwpI/nNT/wn+Hvx4DxrzWU6wF+P8kl77UwPYlTA7GsT+T7eKGVH8xsxmK8pt6lg\nsvlBA5XosWBWUCGLgcBkAY5e4ZWbkdd183o+oMo78id6C+PQPE66PLDtHWfpRRmN\nm6XC03x6NVhnfvfozoWnmS4+e4qj4F/emCHvn0GMywKBgQDLXlj7YPFVXxZpUvg/\nrheVcCTGbNmQJ+4cZXx87huqwqKgkmtOyeWsRc7zYInYgraDrtCuDBCfP//ZzOh0\nLxepYLTPk5eNn/GT+VVrqsy35Ccr60g7Lp/bzb1WxyhcLbo0KX7/6jl0lP+VKtdv\nmto+4mbSBXSM1Y5BVVoVgJ3T/wKBgQDsiSvPRzVi5TTj13x67PFymTMx3HCe2WzH\nJUyepCmVhTm482zW95pv6raDr5CTO6OYpHtc5sTTRhVYEZoEYFTM9Vw8faBtluWG\nBjkRh4cIpoIARMn74YZKj0C/0vdX7SHdyBOU3bgRPHg08Hwu3xReqT1kEPSI/B2V\n4pe5fVrucwKBgQCNFgUxUA3dJjyMES18MDDYUZaRug4tfiYouRdmLGIxUxozv6CG\nZnbZzwxFt+GpvPUV4f+P33rgoCvFU+yoPctyjE6j+0aW0DFucPmb2kBwCu5J/856\nkFwCx3blbwFHAco+SdN7g2kcwgmV2MTg/lMOcU7XwUUcN0Obe7UlWbckzQKBgQDQ\nnXaXHL24GGFaZe4y2JFmujmNy1dEsoye44W9ERpf9h1fwsoGmmCKPp90az5+rIXw\nFXl8CUgk8lXW08db/r4r+ma8Lyx0GzcZyplAnaB5/6j+pazjSxfO4KOBy4Y89Tb+\nTP0AOcCi6ws13bgY+sUTa/5qKA4UVw+c5zlb7nRpgwKBgGXAXhenFw1666482iiN\ncHSgwc4ZHa1oL6aNJR1XWH+aboBSwR+feKHUPeT4jHgzRGo/aCNHD2FE5I8eBv33\nof1kWYjAO0YdzeKrW0rTwfvt9gGg+CS397aWu4cy+mTI+MNfBgeDAIVBeJOJXLlX\nhL8bFAuNNVrCOp79TNnNIsh7\n-----END PRIVATE KEY-----\n"  # noqa: E501
PUBLIC_KEY = "-----BEGIN PUBLIC KEY-----\nMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAu+fyUt6zHCycbxYKTsxe\nftoB/mIoN0RNjE5fbsAtAWSgL+n6GK+8csEMliCvltXAYc/EeC3xZmaNozNGgr3U\nZjQXVtBA0k5xy8QrBYaEFi1avmyOX+Y85HKIoqFLWhKQAz6sIFVC1bTf55zyYmev\n3VN9At45kevi1IBJ7VLVrd3qt8UCJ+ZRt8wtZJQWPx8bXx+R7vMQb8EUEyZ4d1KT\ny4/F8Epq4g4Ha4Ue6OrplslbhqzCpSx5nor5X842LX8ztTDiDBvLdv88RkfsW+Fc\nJLO97B9Sq7h/9PyTF1Pe56cKXvlJvrl4aZXT8oBhKxImu2m8mQdoDKV6q1mCjKNN\njQIDAQAB\n-----END PUBLIC KEY-----\n"  # noqa: E501


def test_metadata_credentials():
    credentials = ydb.iam.MetadataUrlCredentials()
    raised = False
    try:
        credentials.auth_metadata()
    except Exception:
        raised = True

    assert raised


class IamTokenServiceForTest(iam_token_service_pb2_grpc.IamTokenServiceServicer):
    def Create(self, request, context):
        print("IAM token service request: {}".format(request))
        # Validate jwt:
        decoded = jwt.decode(
            request.jwt, key=PUBLIC_KEY, algorithms=["PS256"], audience="https://iam.api.cloud.yandex.net/iam/v1/tokens"
        )
        assert decoded["iss"] == SERVICE_ACCOUNT_ID
        assert decoded["aud"] == "https://iam.api.cloud.yandex.net/iam/v1/tokens"
        assert abs(decoded["iat"] - time.time()) <= 60
        assert abs(decoded["exp"] - time.time()) <= 3600

        response = iam_token_service_pb2.CreateIamTokenResponse(iam_token="test_token")
        response.expires_at.seconds = int(time.time() + 42)
        return response


class IamTokenServiceTestServer(object):
    def __init__(self):
        self.server = grpc.server(concurrent.futures.ThreadPoolExecutor(max_workers=2))
        iam_token_service_pb2_grpc.add_IamTokenServiceServicer_to_server(IamTokenServiceForTest(), self.server)
        self.server.add_insecure_port(self.get_endpoint())
        self.server.start()

    def stop(self):
        self.server.stop(1)
        self.server.wait_for_termination()

    def get_endpoint(self):
        return "localhost:54321"


class ServiceAccountCredentialsForTest(ydb.iam.ServiceAccountCredentials):
    def _channel_factory(self):
        return grpc.insecure_channel(self._iam_endpoint)

    def get_expire_time(self):
        return self._expires_in - time.time()


def test_yandex_service_account_credentials():
    server = IamTokenServiceTestServer()
    credentials = ServiceAccountCredentialsForTest(
        SERVICE_ACCOUNT_ID, ACCESS_KEY_ID, PRIVATE_KEY, server.get_endpoint()
    )
    t = credentials.get_auth_token()
    assert t == "test_token"
    assert credentials.get_expire_time() <= 42
    server.stop()
